#!/usr/bin/env python3
import unittest
from anyBSM import anyBSM, loopfunctions
import numpy as np
import numpy.testing as test
from cmath import pi, sqrt
import logging
logging.getLogger('anyBSM').setLevel(logging.WARNING)


THDM = anyBSM('THDMII', evaluation='numerical', caching = 0, quiet = True, progress = False)
THDM.setparameters({'Mh1':125.09, 'SinBmA':0.95,'M':100,'Mh2':300,'MAh2':320,'MHm2':325})

loopfunctions.MSDR = int(not THDM.dimensional_reduction) # legacy

p = THDM.all_particles
THDM.find_SM_particles(names={'u3'})
higgs = p['h1']
Mh = higgs.nmass
v = THDM.parameters['vSM'].nvalue
Mt = THDM.all_particles['u3'].nmass
Mb = THDM.all_particles['d3'].nmass
loopfunctions.set_renscale(Mt)
Mh = higgs.nmass
MZ = THDM.parameters['MZ'].nvalue
MWp = THDM.parameters['MWp'].nvalue
alpha = 1/(THDM._eval('aEWM1'))
EL = sqrt(4*pi*alpha)
CW2 = THDM._eval('MWp/MZ')**2
SW2 = 1-CW2
SBA = THDM.parameters['SinBmA'].nvalue
aH = THDM.parameters['alphaH'].nvalue
bH = THDM.parameters['betaH'].nvalue
CBA = np.cos(np.arcsin(SBA))
CAB = np.cos(aH+bH)
SAB = np.sin(aH+bH)
C2A = np.cos(2*aH)
S2A = np.sin(2*aH)
C2B = np.cos(2*bH)
S2B = np.sin(2*bH)
M = THDM.parameters['M'].nvalue
MH = p['h2'].nmass
MG0 = p['Ah1'].nmass
MA = p['Ah2'].nmass
MGp = p['Hm1'].nmass
MHp = p['Hm2'].nmass
Lam1 = THDM.parameters['Lam1'].nvalue
Lam2 = THDM.parameters['Lam2'].nvalue
Lam3 = THDM.parameters['Lam3'].nvalue
Lam4 = THDM.parameters['Lam4'].nvalue
Lam5 = THDM.parameters['Lam5'].nvalue
ZH11 = THDM.parameters['ZH11'].nvalue
ZH12 = THDM.parameters['ZH12'].nvalue
ZH21 = THDM.parameters['ZH21'].nvalue
ZH22 = THDM.parameters['ZH22'].nvalue
ZA11 = THDM.parameters['ZA11'].nvalue
ZA12 = THDM.parameters['ZA12'].nvalue
ZA21 = THDM.parameters['ZA21'].nvalue
ZA22 = THDM.parameters['ZA22'].nvalue
ZP11 = THDM.parameters['ZP11'].nvalue
ZP12 = THDM.parameters['ZP12'].nvalue
ZP21 = THDM.parameters['ZP21'].nvalue
ZP22 = THDM.parameters['ZP22'].nvalue
tanb = THDM.parameters['TanBeta'].nvalue
v1 = THDM.parameters['v1'].nvalue
v2 = THDM.parameters['v2'].nvalue
lfac = 1/(16*pi**2)

class numericTest(unittest.TestCase):

    def test_tad(self):
        """ test h tadpole """
        htadanyH3 = THDM.process(higgs, exclude_particles=['d1','d2','d3','u1','u2','e1','e2','e3'])
        htadtopFC = - 12*lfac*Mt**2/v2*ZH12*loopfunctions.A0(Mt**2)
        htadsvuFC = - (lfac*(-(((Lam3 + Lam4 - Lam5)*v1*(ZA22*ZA22)*ZH11 + 2*ZA21*(Lam1*v1*ZA21 + Lam5*v2*ZA22)*ZH11 + (Lam3 + Lam4 - Lam5)*v2*(ZA21*ZA21)*ZH12 + 2*ZA22*(Lam5*v1*ZA21 + Lam2*v2*ZA22)*ZH12)*loopfunctions.A0(MA*MA)) - ((Lam3 + Lam4 - Lam5)*v1*(ZA12*ZA12)*ZH11 + 2*ZA11*(Lam1*v1*ZA11 + Lam5*v2*ZA12)*ZH11 + (Lam3 + Lam4 - Lam5)*v2*(ZA11*ZA11)*ZH12 + 2*ZA12*(Lam5*v1*ZA11 + Lam2*v2*ZA12)*ZH12)*loopfunctions.A0(MG0*MG0) - 3*((Lam3 + Lam4 + Lam5)*ZH11*ZH12*(v2*ZH11 + v1*ZH12) + 2*(Lam1*v1*ZH11**3 + Lam2*v2*ZH12**3))*loopfunctions.A0(Mh*Mh) - ((6*Lam1*v1*ZH11 + (Lam3 + Lam4 + Lam5)*v2*ZH12)*(ZH21*ZH21) + 2*(Lam3 + Lam4 + Lam5)*(v2*ZH11 + v1*ZH12)*ZH21*ZH22 + ((Lam3 + Lam4 + Lam5)*v1*ZH11 + 6*Lam2*v2*ZH12)*(ZH22*ZH22))*loopfunctions.A0(MH**2) - 2*(((2*Lam1*v1*ZH11 + Lam3*v2*ZH12)*(ZP11*ZP11) + (Lam4 + Lam5)*(v2*ZH11 + v1*ZH12)*ZP11*ZP12 + (Lam3*v1*ZH11 + 2*Lam2*v2*ZH12)*(ZP12*ZP12))*loopfunctions.A0(MGp*MGp) + ((2*Lam1*v1*ZH11 + Lam3*v2*ZH12)*(ZP21*ZP21) + (Lam4 + Lam5)*(v2*ZH11 + v1*ZH12)*ZP21*ZP22 + (Lam3*v1*ZH11 + 2*Lam2*v2*ZH12)*(ZP22*ZP22))*loopfunctions.A0(MHp**2)) + (2*alpha*pi*(v1*ZH11 + v2*ZH12)*(loopfunctions.MSDR*(CW2*CW2*MZ**2 + MZ**2*SW2*(1 + SW2) + CW2*(4*(MWp*MWp) + MZ**2 + 2*MZ**2*SW2)) - 6*CW2*loopfunctions.A0(MWp*MWp) - 3*loopfunctions.A0(MZ**2)))/(CW2*SW2)))/2
        test.assert_allclose(htadanyH3['OnePointA'], htadtopFC+htadsvuFC)

    """ test test hh self-energy """
    def test_se_hh_A(self):
        hhselfsAanyH3 = THDM.process(higgs,higgs,momenta=[Mh**2],exclude_particles=['d1','d2','d3','u1','u2','e1','e2','e3'], only_topologies=['TwoPointA'])['TwoPointA']
        hhselfAsFC = - 3*lfac*(Lam1*ZH11**4 + (Lam3 + Lam4 + Lam5)*(ZH11*ZH11)*(ZH12*ZH12) + Lam2*ZH12**4)*loopfunctions.A0(Mh**2)-(lfac*(((2*Lam1*(ZA21*ZA21) + (Lam3 + Lam4 - Lam5)*(ZA22*ZA22))*(ZH11*ZH11) + 4*Lam5*ZA21*ZA22*ZH11*ZH12 + ((Lam3 + Lam4 - Lam5)*(ZA21*ZA21) + 2*Lam2*(ZA22*ZA22))*(ZH12*ZH12))*loopfunctions.A0(MA*MA) + ((2*Lam1*(ZA11*ZA11) + (Lam3 + Lam4 - Lam5)*(ZA12*ZA12))*(ZH11*ZH11) + 4*Lam5*ZA11*ZA12*ZH11*ZH12 + ((Lam3 + Lam4 - Lam5)*(ZA11*ZA11) + 2*Lam2*(ZA12*ZA12))*(ZH12*ZH12))*loopfunctions.A0(MG0*MG0) + ((6*Lam1*(ZH11*ZH11) + (Lam3 + Lam4 + Lam5)*(ZH12*ZH12))*(ZH21*ZH21) + 4*(Lam3 + Lam4 + Lam5)*ZH11*ZH12*ZH21*ZH22 + ((Lam3 + Lam4 + Lam5)*(ZH11*ZH11) + 6*Lam2*(ZH12*ZH12))*(ZH22*ZH22))*loopfunctions.A0(MH**2)))/2 - lfac*(((2*Lam1*(ZH11*ZH11) + Lam3*(ZH12*ZH12))*(ZP11*ZP11) + 2*(Lam4 + Lam5)*ZH11*ZH12*ZP11*ZP12 + (Lam3*(ZH11*ZH11) + 2*Lam2*(ZH12*ZH12))*(ZP12*ZP12))*loopfunctions.A0(MGp*MGp) + ((2*Lam1*(ZH11*ZH11) + Lam3*(ZH12*ZH12))*(ZP21*ZP21) + 2*(Lam4 + Lam5)*ZH11*ZH12*ZP21*ZP22 + (Lam3*(ZH11*ZH11) + 2*Lam2*(ZH12*ZH12))*(ZP22*ZP22))*loopfunctions.A0(MHp**2))
        hhselfAvFC = (alpha*lfac*pi*(ZH11*ZH11 + ZH12*ZH12)*(loopfunctions.MSDR*(CW2*CW2*MZ**2 + MZ**2*SW2*(1 + SW2) + CW2*(4*(MWp*MWp) + MZ**2 + 2*MZ**2*SW2)) - 8*CW2*loopfunctions.A0(MWp*MWp) - 4*loopfunctions.A0(MZ**2)))/(CW2*SW2)
        test.assert_allclose(hhselfsAanyH3, hhselfAsFC+hhselfAvFC)

    def test_se_hh_B(self):
        hhselfsBanyH3 = THDM.process(higgs,higgs,momenta=[Mh**2],exclude_particles=['d1','d2','d3','u1','u2','e1','e2','e3'], only_topologies=['TwoPointB'])['TwoPointB']
        hhselfBtopFC = (12*lfac*(Mt*Mt)*(ZH12*ZH12)*(loopfunctions.A0(Mt*Mt) + 2*(Mt*Mt)*loopfunctions.B0(Mh**2,Mt*Mt,Mt*Mt) + Mh**2*loopfunctions.B1(Mh**2,Mt*Mt,Mt*Mt)))/(v2*v2)
        hhselfBuFC = (alpha**2*lfac*(pi**2)*(v1*ZH11 + v2*ZH12)**2*(2*(CW2*CW2)*loopfunctions.B0(Mh*Mh,MWp*MWp,MWp*MWp) + loopfunctions.B0(Mh*Mh,MZ**2,MZ**2)))/(CW2*CW2*(SW2*SW2))
        hhselfBZFC = 64*alpha**2*lfac*(pi*pi)*(v1*ZH11 + v2*ZH12)**2*(loopfunctions.MSDR - 2*loopfunctions.B0(Mh*Mh,MZ**2,MZ**2))*(1/np.sin(2*np.arccos(sqrt(CW2))))**4
        hhselfBWFC = (8*alpha**2*lfac*(pi*pi)*(v1*ZH11 + v2*ZH12)**2*(loopfunctions.MSDR - 2*loopfunctions.B0(Mh*Mh,MWp*MWp,MWp*MWp)))/(SW2*SW2)
        hhselfBvFC = hhselfBZFC+hhselfBWFC
        hhselfBsvFC = (alpha*lfac*pi*(((pow(ZA11*ZH11 + ZA12*ZH12,2) + pow(ZA21*ZH11 + ZA22*ZH12,2))*loopfunctions.A0(MZ**2) + (MA*MA + 3*(Mh*Mh))*pow(ZA21*ZH11 + ZA22*ZH12,2)*loopfunctions.B0(Mh*Mh,MA*MA,MZ**2) + (MG0*MG0 + 3*(Mh*Mh))*pow(ZA11*ZH11 + ZA12*ZH12,2)*loopfunctions.B0(Mh*Mh,MG0*MG0,MZ**2)) + 2*(CW2)*((pow(ZH11*ZP11 + ZH12*ZP12,2) + pow(ZH11*ZP21 + ZH12*ZP22,2))*loopfunctions.A0(MWp*MWp) + (MGp*MGp + 3*(Mh*Mh))*pow(ZH11*ZP11 + ZH12*ZP12,2)*loopfunctions.B0(Mh*Mh,MGp*MGp,MWp*MWp) + (3*(Mh*Mh) + MHp**2)*pow(ZH11*ZP21 + ZH12*ZP22,2)*loopfunctions.B0(Mh*Mh,MHp**2,MWp*MWp)) + 2*(Mh*Mh)*(2*(CW2)*(pow(ZH11*ZP11 + ZH12*ZP12,2)*loopfunctions.B1(Mh*Mh,MWp*MWp,MGp*MGp) + pow(ZH11*ZP21 + ZH12*ZP22,2)*loopfunctions.B1(Mh*Mh,MWp*MWp,MHp**2)) + (pow(ZA21*ZH11 + ZA22*ZH12,2)*loopfunctions.B1(Mh*Mh,MZ**2,MA*MA) + pow(ZA11*ZH11 + ZA12*ZH12,2)*loopfunctions.B1(Mh*Mh,MZ**2,MG0*MG0)))))/(CW2*(SW2))
        hhselfBh1h2FC = - (lfac*(9*pow((Lam3 + Lam4 + Lam5)*ZH11*ZH12*(v2*ZH11 + v1*ZH12) + 2*(Lam1*v1*pow(ZH11,3) + Lam2*v2*pow(ZH12,3)),2)*loopfunctions.B0(Mh*Mh,Mh*Mh,Mh*Mh) + 2*pow(6*Lam1*v1*(ZH11*ZH11)*ZH21 + (Lam3 + Lam4 + Lam5)*ZH12*(2*v2*ZH11 + v1*ZH12)*ZH21 + 6*Lam2*v2*(ZH12*ZH12)*ZH22 + (Lam3 + Lam4 + Lam5)*ZH11*(v2*ZH11 + 2*v1*ZH12)*ZH22,2)*loopfunctions.B0(Mh*Mh,Mh*Mh,MH**2) + pow((6*Lam1*v1*ZH11 + (Lam3 + Lam4 + Lam5)*v2*ZH12)*(ZH21*ZH21) + 2*(Lam3 + Lam4 + Lam5)*(v2*ZH11 + v1*ZH12)*ZH21*ZH22 + ((Lam3 + Lam4 + Lam5)*v1*ZH11 + 6*Lam2*v2*ZH12)*(ZH22*ZH22),2)*loopfunctions.B0(Mh*Mh,MH**2,MH**2)))/2.
        hhselfBAh1Ah2FC = - (lfac*pow((2*Lam1*v1*ZA11 + Lam5*v2*ZA12)*ZA21*ZH11 + (Lam5*v2*ZA11 + (Lam3 + Lam4 - Lam5)*v1*ZA12)*ZA22*ZH11 + Lam5*v1*(ZA12*ZA21 + ZA11*ZA22)*ZH12 + v2*((Lam3 + Lam4 - Lam5)*ZA11*ZA21 + 2*Lam2*ZA12*ZA22)*ZH12,2)*loopfunctions.B0(Mh*Mh,MA*MA,MG0*MG0)) - (lfac*(pow((Lam3 + Lam4 - Lam5)*v1*(ZA22*ZA22)*ZH11 + 2*ZA21*(Lam1*v1*ZA21 + Lam5*v2*ZA22)*ZH11 + (Lam3 + Lam4 - Lam5)*v2*(ZA21*ZA21)*ZH12 + 2*ZA22*(Lam5*v1*ZA21 + Lam2*v2*ZA22)*ZH12,2)*loopfunctions.B0(Mh*Mh,MA*MA,MA*MA) + pow((Lam3 + Lam4 - Lam5)*v1*(ZA12*ZA12)*ZH11 + 2*ZA11*(Lam1*v1*ZA11 + Lam5*v2*ZA12)*ZH11 + (Lam3 + Lam4 - Lam5)*v2*(ZA11*ZA11)*ZH12 + 2*ZA12*(Lam5*v1*ZA11 + Lam2*v2*ZA12)*ZH12,2)*loopfunctions.B0(Mh*Mh,MG0*MG0,MG0*MG0)))/2.
        hhselfBHm1Hm2FC = - (lfac*pow(2*(2*Lam1*v1*ZH11 + Lam3*v2*ZH12)*ZP11*ZP21 + (Lam4 + Lam5)*(v2*ZH11 + v1*ZH12)*ZP12*ZP21 + v1*((Lam4 + Lam5)*ZH12*ZP11 + 2*Lam3*ZH11*ZP12)*ZP22 + v2*((Lam4 + Lam5)*ZH11*ZP11 + 4*Lam2*ZH12*ZP12)*ZP22,2)*loopfunctions.B0(Mh*Mh,MGp*MGp,MHp**2))/2. - lfac*(pow((2*Lam1*v1*ZH11 + Lam3*v2*ZH12)*(ZP11*ZP11) + (Lam4 + Lam5)*v2*ZH11*ZP11*ZP12 + (Lam4 + Lam5)*v1*ZH12*ZP11*ZP12 + Lam3*v1*ZH11*(ZP12*ZP12) + 2*Lam2*v2*ZH12*(ZP12*ZP12),2)*loopfunctions.B0(Mh*Mh,MGp*MGp,MGp*MGp) + pow((2*Lam1*v1*ZH11 + Lam3*v2*ZH12)*(ZP21*ZP21) + (Lam4 + Lam5)*v2*ZH11*ZP21*ZP22 + (Lam4 + Lam5)*v1*ZH12*ZP21*ZP22 + Lam3*v1*ZH11*(ZP22*ZP22) + 2*Lam2*v2*ZH12*(ZP22*ZP22),2)*loopfunctions.B0(Mh*Mh,MHp**2,MHp**2))
        hhselfBsFC = hhselfBh1h2FC+hhselfBAh1Ah2FC+hhselfBHm1Hm2FC
        hhselfBtotFC = hhselfBsFC+hhselfBsvFC+hhselfBvFC+hhselfBuFC+hhselfBtopFC
        test.assert_allclose(hhselfsBanyH3, hhselfBtotFC)

    def test_se_hh_tad(self):
        hhselfsanyH3tad = THDM.process(higgs,higgs,momenta=[Mh**2],exclude_particles=['d1','d2','d3','u1','u2','e1','e2','e3'], only_topologies=['TwoPointTA'])['TwoPointTA']
        hhselfTAtopFC = - 6*sqrt(2.)*lfac*Mt*(3*ZH12*((Lam3 + Lam4 + Lam5)*ZH11*ZH12*(v2*ZH11 + v1*ZH12) + 2*(Lam1*v1*ZH11**3 + Lam2*v2*ZH12**3))/Mh**2 + ZH22*(6*Lam1*v1*(ZH11*ZH11)*ZH21 + (Lam3 + Lam4 + Lam5)*ZH12*(2*v2*ZH11 + v1*ZH12)*ZH21 + 6*Lam2*v2*(ZH12*ZH12)*ZH22 + (Lam3 + Lam4 + Lam5)*ZH11*(v2*ZH11 + 2*v1*ZH12)*ZH22)/MH**2)*loopfunctions.A0(Mt*Mt)*sqrt(2)*Mt/v2
        hhselfTAsvuFC = (lfac*(3*((Lam3 + Lam4 + Lam5)*ZH11*ZH12*(v2*ZH11 + v1*ZH12) + 2*(Lam1*v1*ZH11**3 + Lam2*v2*ZH12**3))/Mh**2*(3*CW2*SW2*((Lam3 + Lam4 + Lam5)*ZH11*ZH12*(v2*ZH11 + v1*ZH12) + 2*(Lam1*v1*ZH11**3 + Lam2*v2*ZH12**3))*loopfunctions.A0(Mh*Mh) + CW2*SW2*(((Lam3 + Lam4 - Lam5)*v1*(ZA22*ZA22)*ZH11 + 2*ZA21*(Lam1*v1*ZA21 + Lam5*v2*ZA22)*ZH11 + (Lam3 + Lam4 - Lam5)*v2*(ZA21*ZA21)*ZH12 + 2*ZA22*(Lam5*v1*ZA21 + Lam2*v2*ZA22)*ZH12)*loopfunctions.A0(MA*MA) + ((Lam3 + Lam4 - Lam5)*v1*(ZA12*ZA12)*ZH11 + 2*ZA11*(Lam1*v1*ZA11 + Lam5*v2*ZA12)*ZH11 + (Lam3 + Lam4 - Lam5)*v2*(ZA11*ZA11)*ZH12 + 2*ZA12*(Lam5*v1*ZA11 + Lam2*v2*ZA12)*ZH12)*loopfunctions.A0(MG0*MG0) + ((6*Lam1*v1*ZH11 + (Lam3 + Lam4 + Lam5)*v2*ZH12)*(ZH21*ZH21) + 2*(Lam3 + Lam4 + Lam5)*(v2*ZH11 + v1*ZH12)*ZH21*ZH22 + ((Lam3 + Lam4 + Lam5)*v1*ZH11 + 6*Lam2*v2*ZH12)*(ZH22*ZH22))*loopfunctions.A0(MH**2)) + 2*CW2*SW2*(((2*Lam1*v1*ZH11 + Lam3*v2*ZH12)*(ZP11*ZP11) + (Lam4 + Lam5)*(v2*ZH11 + v1*ZH12)*ZP11*ZP12 + (Lam3*v1*ZH11 + 2*Lam2*v2*ZH12)*(ZP12*ZP12))*loopfunctions.A0(MGp*MGp) + ((2*Lam1*v1*ZH11 + Lam3*v2*ZH12)*(ZP21*ZP21) + (Lam4 + Lam5)*(v2*ZH11 + v1*ZH12)*ZP21*ZP22 + (Lam3*v1*ZH11 + 2*Lam2*v2*ZH12)*(ZP22*ZP22))*loopfunctions.A0(MHp**2)) - alpha*pi*(v1*ZH11 + v2*ZH12)*(2*loopfunctions.MSDR*(CW2*CW2*MZ**2 + MZ**2*SW2*(1 + SW2) + CW2*(4*(MWp*MWp) + MZ**2 + 2*MZ**2*SW2)) - 12*CW2*loopfunctions.A0(MWp*MWp) - 6*loopfunctions.A0(MZ**2))) + (6*Lam1*v1*(ZH11*ZH11)*ZH21 + (Lam3 + Lam4 + Lam5)*ZH12*(2*v2*ZH11 + v1*ZH12)*ZH21 + 6*Lam2*v2*(ZH12*ZH12)*ZH22 + (Lam3 + Lam4 + Lam5)*ZH11*(v2*ZH11 + 2*v1*ZH12)*ZH22)/MH**2*(CW2*SW2*(((Lam3 + Lam4 - Lam5)*v1*(ZA22*ZA22)*ZH21 + 2*ZA21*(Lam1*v1*ZA21 + Lam5*v2*ZA22)*ZH21 + (Lam3 + Lam4 - Lam5)*v2*(ZA21*ZA21)*ZH22 + 2*ZA22*(Lam5*v1*ZA21 + Lam2*v2*ZA22)*ZH22)*loopfunctions.A0(MA*MA) + ((Lam3 + Lam4 - Lam5)*v1*(ZA12*ZA12)*ZH21 + 2*ZA11*(Lam1*v1*ZA11 + Lam5*v2*ZA12)*ZH21 + (Lam3 + Lam4 - Lam5)*v2*(ZA11*ZA11)*ZH22 + 2*ZA12*(Lam5*v1*ZA11 + Lam2*v2*ZA12)*ZH22)*loopfunctions.A0(MG0*MG0) + (6*Lam1*v1*(ZH11*ZH11)*ZH21 + (Lam3 + Lam4 + Lam5)*ZH12*(2*v2*ZH11 + v1*ZH12)*ZH21 + 6*Lam2*v2*(ZH12*ZH12)*ZH22 + (Lam3 + Lam4 + Lam5)*ZH11*(v2*ZH11 + 2*v1*ZH12)*ZH22)*loopfunctions.A0(Mh*Mh)) + 3*CW2*SW2*((Lam3 + Lam4 + Lam5)*ZH21*ZH22*(v2*ZH21 + v1*ZH22) + 2*(Lam1*v1*ZH21**3 + Lam2*v2*ZH22**3))*loopfunctions.A0(MH**2) + 2*CW2*SW2*(((2*Lam1*v1*ZH21 + Lam3*v2*ZH22)*(ZP11*ZP11) + (Lam4 + Lam5)*(v2*ZH21 + v1*ZH22)*ZP11*ZP12 + (Lam3*v1*ZH21 + 2*Lam2*v2*ZH22)*(ZP12*ZP12))*loopfunctions.A0(MGp*MGp) + ((2*Lam1*v1*ZH21 + Lam3*v2*ZH22)*(ZP21*ZP21) + (Lam4 + Lam5)*(v2*ZH21 + v1*ZH22)*ZP21*ZP22 + (Lam3*v1*ZH21 + 2*Lam2*v2*ZH22)*(ZP22*ZP22))*loopfunctions.A0(MHp**2)) - alpha*pi*(v1*ZH21 + v2*ZH22)*(2*loopfunctions.MSDR*(CW2*CW2*MZ**2 + MZ**2*SW2*(1 + SW2) + CW2*(4*(MWp*MWp) + MZ**2 + 2*MZ**2*SW2)) - 12*CW2*loopfunctions.A0(MWp*MWp) - 6*loopfunctions.A0(MZ**2)))))/(2.*CW2*SW2)
        test.assert_allclose(hhselfsanyH3tad, hhselfTAtopFC+hhselfTAsvuFC)

    """ test hH mixing self-energy """
    def test_se_hH_A(self):
        for pval in (150, 300, 1500, Mh):
            hHselfsAanyH3 = THDM.process(higgs,p['h2'],momenta=[pval**2],exclude_particles=['d1','d2','d3','u1','u2','e1','e2','e3'], only_topologies=['TwoPointA'])['TwoPointA']
            hHselfAFC = (lfac*(-(((2*Lam1*(ZA21*ZA21) + (Lam3 + Lam4 - Lam5)*(ZA22*ZA22))*ZH11*ZH21 + ((Lam3 + Lam4 - Lam5)*(ZA21*ZA21) + 2*Lam2*(ZA22*ZA22))*ZH12*ZH22 + 2*Lam5*ZA21*ZA22*(ZH12*ZH21 + ZH11*ZH22))*loopfunctions.A0(MA*MA)) - ((2*Lam1*(ZA11*ZA11) + (Lam3 + Lam4 - Lam5)*(ZA12*ZA12))*ZH11*ZH21 + ((Lam3 + Lam4 - Lam5)*(ZA11*ZA11) + 2*Lam2*(ZA12*ZA12))*ZH12*ZH22 + 2*Lam5*ZA11*ZA12*(ZH12*ZH21 + ZH11*ZH22))*loopfunctions.A0(MG0*MG0) - 3*((2*Lam1*pow(ZH11,3)*ZH21 + (Lam3 + Lam4 + Lam5)*ZH11*(ZH12*ZH12)*ZH21 + (Lam3 + Lam4 + Lam5)*(ZH11*ZH11)*ZH12*ZH22 + 2*Lam2*pow(ZH12,3)*ZH22)*loopfunctions.A0(Mh*Mh) + ((Lam3 + Lam4 + Lam5)*ZH21*ZH22*(ZH12*ZH21 + ZH11*ZH22) + 2*(Lam1*ZH11*pow(ZH21,3) + Lam2*ZH12*pow(ZH22,3)))*loopfunctions.A0(MH**2)) - 2*(((2*Lam1*ZH11*ZH21 + Lam3*ZH12*ZH22)*(ZP11*ZP11) + (Lam4 + Lam5)*ZH12*ZH21*ZP11*ZP12 + (Lam4 + Lam5)*ZH11*ZH22*ZP11*ZP12 + Lam3*ZH11*ZH21*(ZP12*ZP12) + 2*Lam2*ZH12*ZH22*(ZP12*ZP12))*loopfunctions.A0(MGp*MGp) + ((2*Lam1*ZH11*ZH21 + Lam3*ZH12*ZH22)*(ZP21*ZP21) + (Lam4 + Lam5)*ZH12*ZH21*ZP21*ZP22 + (Lam4 + Lam5)*ZH11*ZH22*ZP21*ZP22 + Lam3*ZH11*ZH21*(ZP22*ZP22) + 2*Lam2*ZH12*ZH22*(ZP22*ZP22))*loopfunctions.A0(MHp**2)) + (2*alpha*pi*(ZH11*ZH21 + ZH12*ZH22)*(loopfunctions.MSDR*(CW2*CW2*MZ**2 + MZ**2*SW2*(1 + SW2) + CW2*(4*(MWp*MWp) + MZ**2 + 2*MZ**2*SW2)) - 8*CW2*loopfunctions.A0(MWp*MWp) - 2*(1 + CW2*CW2 + 2*CW2*SW2 + SW2*SW2)*loopfunctions.A0(MZ**2)))/(CW2*SW2)))/2.
            test.assert_allclose(hHselfsAanyH3, hHselfAFC)

    def test_se_hH_B(self):
        for pval in (150, 300, 1500, Mh):
            hHselfsBanyH3 = THDM.process(higgs,p['h2'],momenta=[pval**2],exclude_particles=['d1','d2','d3','u1','u2','e1','e2','e3'], only_topologies=['TwoPointB'])['TwoPointB']
            hHselfBtopFC = (-2*lfac*(Mt*Mt)*ZH12*ZH22*(-6*loopfunctions.A0(Mt*Mt) - 6*(2*(Mt*Mt)*loopfunctions.B0(pval*pval,Mt*Mt,Mt*Mt) + pval*pval*loopfunctions.B1(pval*pval,Mt*Mt,Mt*Mt))))/(v2*v2)
            hHselfBuFC = (alpha**2*lfac*(pi*pi)*(v1*ZH11 + v2*ZH12)*(v1*ZH21 + v2*ZH22)*(8*(CW2*CW2)*loopfunctions.B0(pval*pval,MWp*MWp,MWp*MWp) + pow(1 + CW2*CW2 + 2*CW2*SW2 + SW2*SW2,2)*loopfunctions.B0(pval*pval,MZ**2,MZ**2)))/(4.*(CW2*CW2)*(SW2*SW2))
            hHselfBsFC = - (lfac*(((Lam3 + Lam4 - Lam5)*v1*(ZA22*ZA22)*ZH11 + 2*ZA21*(Lam1*v1*ZA21 + Lam5*v2*ZA22)*ZH11 + (Lam3 + Lam4 - Lam5)*v2*(ZA21*ZA21)*ZH12 + 2*ZA22*(Lam5*v1*ZA21 + Lam2*v2*ZA22)*ZH12)*((Lam3 + Lam4 - Lam5)*v1*(ZA22*ZA22)*ZH21 + 2*ZA21*(Lam1*v1*ZA21 + Lam5*v2*ZA22)*ZH21 + (Lam3 + Lam4 - Lam5)*v2*(ZA21*ZA21)*ZH22 + 2*ZA22*(Lam5*v1*ZA21 + Lam2*v2*ZA22)*ZH22)*loopfunctions.B0(pval*pval,MA*MA,MA*MA) + ((Lam3 + Lam4 - Lam5)*v1*(ZA12*ZA12)*ZH11 + 2*ZA11*(Lam1*v1*ZA11 + Lam5*v2*ZA12)*ZH11 + (Lam3 + Lam4 - Lam5)*v2*(ZA11*ZA11)*ZH12 + 2*ZA12*(Lam5*v1*ZA11 + Lam2*v2*ZA12)*ZH12)*((Lam3 + Lam4 - Lam5)*v1*(ZA12*ZA12)*ZH21 + 2*ZA11*(Lam1*v1*ZA11 + Lam5*v2*ZA12)*ZH21 + (Lam3 + Lam4 - Lam5)*v2*(ZA11*ZA11)*ZH22 + 2*ZA12*(Lam5*v1*ZA11 + Lam2*v2*ZA12)*ZH22)*loopfunctions.B0(pval*pval,MG0*MG0,MG0*MG0) + (2*(2*Lam1*v1*ZH11 + Lam3*v2*ZH12)*ZP11*ZP21 + (Lam4 + Lam5)*(v2*ZH11 + v1*ZH12)*ZP12*ZP21 + v1*((Lam4 + Lam5)*ZH12*ZP11 + 2*Lam3*ZH11*ZP12)*ZP22 + v2*((Lam4 + Lam5)*ZH11*ZP11 + 4*Lam2*ZH12*ZP12)*ZP22)*(2*(2*Lam1*v1*ZH21 + Lam3*v2*ZH22)*ZP11*ZP21 + (Lam4 + Lam5)*(v2*ZH21 + v1*ZH22)*ZP12*ZP21 + v1*((Lam4 + Lam5)*ZH22*ZP11 + 2*Lam3*ZH21*ZP12)*ZP22 + v2*((Lam4 + Lam5)*ZH21*ZP11 + 4*Lam2*ZH22*ZP12)*ZP22)*loopfunctions.B0(pval*pval,MGp*MGp,MHp**2) + (6*Lam1*v1*(ZH11*ZH11)*ZH21 + (Lam3 + Lam4 + Lam5)*ZH12*(2*v2*ZH11 + v1*ZH12)*ZH21 + 6*Lam2*v2*(ZH12*ZH12)*ZH22 + (Lam3 + Lam4 + Lam5)*ZH11*(v2*ZH11 + 2*v1*ZH12)*ZH22)*(3*((Lam3 + Lam4 + Lam5)*ZH11*ZH12*(v2*ZH11 + v1*ZH12) + 2*(Lam1*v1*pow(ZH11,3) + Lam2*v2*pow(ZH12,3)))*loopfunctions.B0(pval*pval,Mh*Mh,Mh*Mh) + 2*((6*Lam1*v1*ZH11 + (Lam3 + Lam4 + Lam5)*v2*ZH12)*(ZH21*ZH21) + 2*(Lam3 + Lam4 + Lam5)*(v2*ZH11 + v1*ZH12)*ZH21*ZH22 + ((Lam3 + Lam4 + Lam5)*v1*ZH11 + 6*Lam2*v2*ZH12)*(ZH22*ZH22))*loopfunctions.B0(pval*pval,Mh*Mh,MH**2)) + 3*((6*Lam1*v1*ZH11 + (Lam3 + Lam4 + Lam5)*v2*ZH12)*(ZH21*ZH21) + 2*(Lam3 + Lam4 + Lam5)*(v2*ZH11 + v1*ZH12)*ZH21*ZH22 + ((Lam3 + Lam4 + Lam5)*v1*ZH11 + 6*Lam2*v2*ZH12)*(ZH22*ZH22))*((Lam3 + Lam4 + Lam5)*ZH21*ZH22*(v2*ZH21 + v1*ZH22) + 2*(Lam1*v1*pow(ZH21,3) + Lam2*v2*pow(ZH22,3)))*loopfunctions.B0(pval*pval,MH**2,MH**2) + 2*(((2*Lam1*v1*ZA11 + Lam5*v2*ZA12)*ZA21*ZH11 + (Lam5*v2*ZA11 + (Lam3 + Lam4 - Lam5)*v1*ZA12)*ZA22*ZH11 + Lam5*v1*(ZA12*ZA21 + ZA11*ZA22)*ZH12 + v2*((Lam3 + Lam4 - Lam5)*ZA11*ZA21 + 2*Lam2*ZA12*ZA22)*ZH12)*((2*Lam1*v1*ZA11 + Lam5*v2*ZA12)*ZA21*ZH21 + (Lam5*v2*ZA11 + (Lam3 + Lam4 - Lam5)*v1*ZA12)*ZA22*ZH21 + Lam5*v1*(ZA12*ZA21 + ZA11*ZA22)*ZH22 + v2*((Lam3 + Lam4 - Lam5)*ZA11*ZA21 + 2*Lam2*ZA12*ZA22)*ZH22)*loopfunctions.B0(pval*pval,MA*MA,MG0*MG0) + ((2*Lam1*v1*ZH11 + Lam3*v2*ZH12)*(ZP11*ZP11) + (Lam4 + Lam5)*v2*ZH11*ZP11*ZP12 + (Lam4 + Lam5)*v1*ZH12*ZP11*ZP12 + Lam3*v1*ZH11*(ZP12*ZP12) + 2*Lam2*v2*ZH12*(ZP12*ZP12))*((2*Lam1*v1*ZH21 + Lam3*v2*ZH22)*(ZP11*ZP11) + (Lam4 + Lam5)*v2*ZH21*ZP11*ZP12 + (Lam4 + Lam5)*v1*ZH22*ZP11*ZP12 + Lam3*v1*ZH21*(ZP12*ZP12) + 2*Lam2*v2*ZH22*(ZP12*ZP12))*loopfunctions.B0(pval*pval,MGp*MGp,MGp*MGp) + ((2*Lam1*v1*ZH11 + Lam3*v2*ZH12)*(ZP21*ZP21) + (Lam4 + Lam5)*v2*ZH11*ZP21*ZP22 + (Lam4 + Lam5)*v1*ZH12*ZP21*ZP22 + Lam3*v1*ZH11*(ZP22*ZP22) + 2*Lam2*v2*ZH12*(ZP22*ZP22))*((2*Lam1*v1*ZH21 + Lam3*v2*ZH22)*(ZP21*ZP21) + (Lam4 + Lam5)*v2*ZH21*ZP21*ZP22 + (Lam4 + Lam5)*v1*ZH22*ZP21*ZP22 + Lam3*v1*ZH21*(ZP22*ZP22) + 2*Lam2*v2*ZH22*(ZP22*ZP22))*loopfunctions.B0(pval*pval,MHp**2,MHp**2))))/2.
            hHselfBvFC = (alpha**2*lfac*(pi*pi)*(v1*ZH11 + v2*ZH12)*(v1*ZH21 + v2*ZH22)*(loopfunctions.MSDR*(pow(CW2,4) + SW2*SW2*pow(1 + SW2,2) + pow(CW2,3)*(2 + 4*SW2) + 2*CW2*SW2*(1 + 3*SW2 + 2*(SW2*SW2)) + CW2*CW2*(9 + 6*SW2 + 6*(SW2*SW2))) - 16*(CW2*CW2)*loopfunctions.B0(pval*pval,MWp*MWp,MWp*MWp) - 2*pow(1 + CW2*CW2 + 2*CW2*SW2 + SW2*SW2,2)*loopfunctions.B0(pval*pval,MZ**2,MZ**2)))/(CW2*CW2*(SW2*SW2))
            hHselfBsvFC = (2*alpha*lfac*pi*(((ZH11*ZP11 + ZH12*ZP12)*(ZH21*ZP11 + ZH22*ZP12) + (ZH11*ZP21 + ZH12*ZP22)*(ZH21*ZP21 + ZH22*ZP22))*loopfunctions.A0(MWp*MWp) + (MGp*MGp + 3*(pval*pval))*(ZH11*ZP11 + ZH12*ZP12)*(ZH21*ZP11 + ZH22*ZP12)*loopfunctions.B0(pval*pval,MGp*MGp,MWp*MWp) + (MHp**2 + 3*(pval*pval))*(ZH11*ZP21 + ZH12*ZP22)*(ZH21*ZP21 + ZH22*ZP22)*loopfunctions.B0(pval*pval,MHp**2,MWp*MWp) + 2*SW2*(((ZA11*ZH11 + ZA12*ZH12)*(ZA11*ZH21 + ZA12*ZH22) + (ZA21*ZH11 + ZA22*ZH12)*(ZA21*ZH21 + ZA22*ZH22))*loopfunctions.A0(MZ**2) + (MA*MA + 3*(pval*pval))*(ZA21*ZH11 + ZA22*ZH12)*(ZA21*ZH21 + ZA22*ZH22)*loopfunctions.B0(pval*pval,MA*MA,MZ**2) + (MG0*MG0 + 3*(pval*pval))*(ZA11*ZH11 + ZA12*ZH12)*(ZA11*ZH21 + ZA12*ZH22)*loopfunctions.B0(pval*pval,MG0*MG0,MZ**2))*pow(1/np.sin(2*np.arccos(sqrt(CW2))),2) + 2*(pval*pval)*((ZH11*ZP11 + ZH12*ZP12)*(ZH21*ZP11 + ZH22*ZP12)*loopfunctions.B1(pval*pval,MWp*MWp,MGp*MGp) + (ZH11*ZP21 + ZH12*ZP22)*(ZH21*ZP21 + ZH22*ZP22)*loopfunctions.B1(pval*pval,MWp*MWp,MHp**2) + 2*SW2*((ZA21*ZH11 + ZA22*ZH12)*(ZA21*ZH21 + ZA22*ZH22)*loopfunctions.B1(pval*pval,MZ**2,MA*MA) + (ZA11*ZH11 + ZA12*ZH12)*(ZA11*ZH21 + ZA12*ZH22)*loopfunctions.B1(pval*pval,MZ**2,MG0*MG0))*pow(1/np.sin(2*np.arccos(sqrt(CW2))),2))))/SW2
            test.assert_allclose(hHselfsBanyH3, hHselfBsFC+hHselfBsvFC+hHselfBvFC+hHselfBuFC+hHselfBtopFC)

    def test_se_hH_tad(self):
        for pval in (150, 300, 1500, Mh):
            hHselfsTAanyH3 = THDM.process(higgs,p['h2'],momenta=[pval**2],exclude_particles=['d1','d2','d3','u1','u2','e1','e2','e3'], only_topologies=['TwoPointTA'])['TwoPointTA']
            hHselfTAtopFC = - 12*lfac*Mt**2/v2*loopfunctions.A0(Mt*Mt)*(ZH12*(6*Lam1*v1*(ZH11*ZH11)*ZH21 + (Lam3 + Lam4 + Lam5)*ZH12*(2*v2*ZH11 + v1*ZH12)*ZH21 + 6*Lam2*v2*(ZH12*ZH12)*ZH22 + (Lam3 + Lam4 + Lam5)*ZH11*(v2*ZH11 + 2*v1*ZH12)*ZH22)/(Mh*Mh) + ZH22*((6*Lam1*v1*ZH11 + (Lam3 + Lam4 + Lam5)*v2*ZH12)*(ZH21*ZH21) + 2*(Lam3 + Lam4 + Lam5)*(v2*ZH11 + v1*ZH12)*ZH21*ZH22 + ((Lam3 + Lam4 + Lam5)*v1*ZH11 + 6*Lam2*v2*ZH12)*(ZH22*ZH22))/(MH**2))
            hHselfTAsvuFC = (lfac*(MH**2*(6*Lam1*v1*(ZH11*ZH11)*ZH21 + (Lam3 + Lam4 + Lam5)*ZH12*(2*v2*ZH11 + v1*ZH12)*ZH21 + 6*Lam2*v2*(ZH12*ZH12)*ZH22 + (Lam3 + Lam4 + Lam5)*ZH11*(v2*ZH11 + 2*v1*ZH12)*ZH22)*(CW2*SW2*(((Lam3 + Lam4 - Lam5)*v1*(ZA22*ZA22)*ZH11 + 2*ZA21*(Lam1*v1*ZA21 + Lam5*v2*ZA22)*ZH11 + (Lam3 + Lam4 - Lam5)*v2*(ZA21*ZA21)*ZH12 + 2*ZA22*(Lam5*v1*ZA21 + Lam2*v2*ZA22)*ZH12)*loopfunctions.A0(MA*MA) + ((Lam3 + Lam4 - Lam5)*v1*(ZA12*ZA12)*ZH11 + 2*ZA11*(Lam1*v1*ZA11 + Lam5*v2*ZA12)*ZH11 + (Lam3 + Lam4 - Lam5)*v2*(ZA11*ZA11)*ZH12 + 2*ZA12*(Lam5*v1*ZA11 + Lam2*v2*ZA12)*ZH12)*loopfunctions.A0(MG0*MG0)) + 3*CW2*SW2*((Lam3 + Lam4 + Lam5)*ZH11*ZH12*(v2*ZH11 + v1*ZH12) + 2*(Lam1*v1*pow(ZH11,3) + Lam2*v2*pow(ZH12,3)))*loopfunctions.A0(Mh*Mh) + CW2*SW2*((6*Lam1*v1*ZH11 + (Lam3 + Lam4 + Lam5)*v2*ZH12)*(ZH21*ZH21) + 2*(Lam3 + Lam4 + Lam5)*(v2*ZH11 + v1*ZH12)*ZH21*ZH22 + ((Lam3 + Lam4 + Lam5)*v1*ZH11 + 6*Lam2*v2*ZH12)*(ZH22*ZH22))*loopfunctions.A0(MH**2) + 2*CW2*SW2*(((2*Lam1*v1*ZH11 + Lam3*v2*ZH12)*(ZP11*ZP11) + (Lam4 + Lam5)*v2*ZH11*ZP11*ZP12 + (Lam4 + Lam5)*v1*ZH12*ZP11*ZP12 + Lam3*v1*ZH11*(ZP12*ZP12) + 2*Lam2*v2*ZH12*(ZP12*ZP12))*loopfunctions.A0(MGp*MGp) + ((2*Lam1*v1*ZH11 + Lam3*v2*ZH12)*(ZP21*ZP21) + (Lam4 + Lam5)*v2*ZH11*ZP21*ZP22 + (Lam4 + Lam5)*v1*ZH12*ZP21*ZP22 + Lam3*v1*ZH11*(ZP22*ZP22) + 2*Lam2*v2*ZH12*(ZP22*ZP22))*loopfunctions.A0(MHp**2)) - alpha*pi*(v1*ZH11 + v2*ZH12)*(2*loopfunctions.MSDR*(CW2*CW2*MZ**2 + MZ**2*SW2*(1 + SW2) + CW2*(4*(MWp*MWp) + MZ**2 + 2*MZ**2*SW2)) - 12*CW2*loopfunctions.A0(MWp*MWp) - 3*(1 + CW2*CW2 + 2*CW2*SW2 + SW2*SW2)*loopfunctions.A0(MZ**2))) + Mh*Mh*((6*Lam1*v1*ZH11 + (Lam3 + Lam4 + Lam5)*v2*ZH12)*(ZH21*ZH21) + 2*(Lam3 + Lam4 + Lam5)*(v2*ZH11 + v1*ZH12)*ZH21*ZH22 + ((Lam3 + Lam4 + Lam5)*v1*ZH11 + 6*Lam2*v2*ZH12)*(ZH22*ZH22))*(CW2*SW2*(((Lam3 + Lam4 - Lam5)*v1*(ZA22*ZA22)*ZH21 + 2*ZA21*(Lam1*v1*ZA21 + Lam5*v2*ZA22)*ZH21 + (Lam3 + Lam4 - Lam5)*v2*(ZA21*ZA21)*ZH22 + 2*ZA22*(Lam5*v1*ZA21 + Lam2*v2*ZA22)*ZH22)*loopfunctions.A0(MA*MA) + ((Lam3 + Lam4 - Lam5)*v1*(ZA12*ZA12)*ZH21 + 2*ZA11*(Lam1*v1*ZA11 + Lam5*v2*ZA12)*ZH21 + (Lam3 + Lam4 - Lam5)*v2*(ZA11*ZA11)*ZH22 + 2*ZA12*(Lam5*v1*ZA11 + Lam2*v2*ZA12)*ZH22)*loopfunctions.A0(MG0*MG0) + (6*Lam1*v1*(ZH11*ZH11)*ZH21 + (Lam3 + Lam4 + Lam5)*ZH12*(2*v2*ZH11 + v1*ZH12)*ZH21 + 6*Lam2*v2*(ZH12*ZH12)*ZH22 + (Lam3 + Lam4 + Lam5)*ZH11*(v2*ZH11 + 2*v1*ZH12)*ZH22)*loopfunctions.A0(Mh*Mh)) + 3*CW2*SW2*((Lam3 + Lam4 + Lam5)*ZH21*ZH22*(v2*ZH21 + v1*ZH22) + 2*(Lam1*v1*pow(ZH21,3) + Lam2*v2*pow(ZH22,3)))*loopfunctions.A0(MH**2) + 2*CW2*SW2*(((2*Lam1*v1*ZH21 + Lam3*v2*ZH22)*(ZP11*ZP11) + (Lam4 + Lam5)*v2*ZH21*ZP11*ZP12 + (Lam4 + Lam5)*v1*ZH22*ZP11*ZP12 + Lam3*v1*ZH21*(ZP12*ZP12) + 2*Lam2*v2*ZH22*(ZP12*ZP12))*loopfunctions.A0(MGp*MGp) + ((2*Lam1*v1*ZH21 + Lam3*v2*ZH22)*(ZP21*ZP21) + (Lam4 + Lam5)*v2*ZH21*ZP21*ZP22 + (Lam4 + Lam5)*v1*ZH22*ZP21*ZP22 + Lam3*v1*ZH21*(ZP22*ZP22) + 2*Lam2*v2*ZH22*(ZP22*ZP22))*loopfunctions.A0(MHp**2)) - alpha*pi*(v1*ZH21 + v2*ZH22)*(2*loopfunctions.MSDR*(CW2*CW2*MZ**2 + MZ**2*SW2*(1 + SW2) + CW2*(4*(MWp*MWp) + MZ**2 + 2*MZ**2*SW2)) - 12*CW2*loopfunctions.A0(MWp*MWp) - 3*(1 + CW2*CW2 + 2*CW2*SW2 + SW2*SW2)*loopfunctions.A0(MZ**2)))))/(2.*CW2*(Mh*Mh)*MH**2*SW2)
            test.assert_allclose(hHselfsTAanyH3, hHselfTAtopFC+hHselfTAsvuFC)

    """ test hH mixing self-energy """
    def test_WFR_A(self):
        wfrtA = THDM.process(higgs,higgs,higgs,momenta=[Mh**2,Mh**2,Mh**2],exclude_particles=['d1','d2','d3','u1','u2','e1','e2','e3'], only_topologies=['ThreePointWFRA'])['ThreePointWFRA']
        lamhhH = - (-((6*Lam1*v1*(ZH11*ZH11) + (Lam3 + Lam4 + Lam5)*ZH12*(2*v2*ZH11 + v1*ZH12))*ZH21) - (6*Lam2*v2*(ZH12*ZH12) + (Lam3 + Lam4 + Lam5)*ZH11*(v2*ZH11 + 2*v1*ZH12))*ZH22)
        hHselfAFC = (lfac*(-(((2*Lam1*(ZA21*ZA21) + (Lam3 + Lam4 - Lam5)*(ZA22*ZA22))*ZH11*ZH21 + ((Lam3 + Lam4 - Lam5)*(ZA21*ZA21) + 2*Lam2*(ZA22*ZA22))*ZH12*ZH22 + 2*Lam5*ZA21*ZA22*(ZH12*ZH21 + ZH11*ZH22))*loopfunctions.A0(MA*MA)) - ((2*Lam1*(ZA11*ZA11) + (Lam3 + Lam4 - Lam5)*(ZA12*ZA12))*ZH11*ZH21 + ((Lam3 + Lam4 - Lam5)*(ZA11*ZA11) + 2*Lam2*(ZA12*ZA12))*ZH12*ZH22 + 2*Lam5*ZA11*ZA12*(ZH12*ZH21 + ZH11*ZH22))*loopfunctions.A0(MG0*MG0) - 3*((2*Lam1*pow(ZH11,3)*ZH21 + (Lam3 + Lam4 + Lam5)*ZH11*(ZH12*ZH12)*ZH21 + (Lam3 + Lam4 + Lam5)*(ZH11*ZH11)*ZH12*ZH22 + 2*Lam2*pow(ZH12,3)*ZH22)*loopfunctions.A0(Mh*Mh) + ((Lam3 + Lam4 + Lam5)*ZH21*ZH22*(ZH12*ZH21 + ZH11*ZH22) + 2*(Lam1*ZH11*pow(ZH21,3) + Lam2*ZH12*pow(ZH22,3)))*loopfunctions.A0(MH**2)) - 2*(((2*Lam1*ZH11*ZH21 + Lam3*ZH12*ZH22)*(ZP11*ZP11) + (Lam4 + Lam5)*ZH12*ZH21*ZP11*ZP12 + (Lam4 + Lam5)*ZH11*ZH22*ZP11*ZP12 + Lam3*ZH11*ZH21*(ZP12*ZP12) + 2*Lam2*ZH12*ZH22*(ZP12*ZP12))*loopfunctions.A0(MGp*MGp) + ((2*Lam1*ZH11*ZH21 + Lam3*ZH12*ZH22)*(ZP21*ZP21) + (Lam4 + Lam5)*ZH12*ZH21*ZP21*ZP22 + (Lam4 + Lam5)*ZH11*ZH22*ZP21*ZP22 + Lam3*ZH11*ZH21*(ZP22*ZP22) + 2*Lam2*ZH12*ZH22*(ZP22*ZP22))*loopfunctions.A0(MHp**2)) + (2*alpha*pi*(ZH11*ZH21 + ZH12*ZH22)*(loopfunctions.MSDR*(CW2*CW2*MZ**2 + MZ**2*SW2*(1 + SW2) + CW2*(4*(MWp*MWp) + MZ**2 + 2*MZ**2*SW2)) - 8*CW2*loopfunctions.A0(MWp*MWp) - 2*(1 + CW2*CW2 + 2*CW2*SW2 + SW2*SW2)*loopfunctions.A0(MZ**2)))/(CW2*SW2)))/2.
        test.assert_allclose(wfrtA,- 3*lamhhH*hHselfAFC/(Mh**2-MH**2))

    def test_WFR_B(self):
        wfrtB = THDM.process(higgs,higgs,higgs,momenta=[Mh**2,Mh**2,Mh**2],exclude_particles=['d1','d2','d3','u1','u2','e1','e2','e3'], only_topologies=['ThreePointWFRB'])['ThreePointWFRB']
        pval = Mh
        lamhhh = - 1/(4*v*np.sin(2*bH))*((np.cos(3*aH-bH)+3*CAB)*Mh**2-4*CBA**2*CAB*M**2)*6
        lamhhH = - (-((6*Lam1*v1*(ZH11*ZH11) + (Lam3 + Lam4 + Lam5)*ZH12*(2*v2*ZH11 + v1*ZH12))*ZH21) - (6*Lam2*v2*(ZH12*ZH12) + (Lam3 + Lam4 + Lam5)*ZH11*(v2*ZH11 + 2*v1*ZH12))*ZH22)
        hhselfBtopFCderiv = (12*lfac*(Mt*Mt)*(ZH12*ZH12)*(loopfunctions.B1(Mh**2,Mt*Mt,Mt*Mt) + 2*(Mt*Mt)*loopfunctions.dB0(Mh**2,Mt*Mt,Mt*Mt) + Mh**2*loopfunctions.dB1(Mh**2,Mt*Mt,Mt*Mt)))/(v2*v2)
        hhselfBuFCderiv = (alpha**2*lfac*(pi*pi)*pow(v1*ZH11 + v2*ZH12,2)*(8*(CW2*CW2)*loopfunctions.dB0(Mh**2,MWp*MWp,MWp*MWp) + pow(1 + CW2*CW2 + 2*CW2*SW2 + SW2*SW2,2)*loopfunctions.dB0(Mh**2,MZ**2,MZ**2)))/(4.*(CW2*CW2)*(SW2*SW2))
        hhselfBsFCderiv = (-9*lfac*pow((Lam3 + Lam4 + Lam5)*ZH11*ZH12*(v2*ZH11 + v1*ZH12) + 2*(Lam1*v1*pow(ZH11,3) + Lam2*v2*pow(ZH12,3)),2)*loopfunctions.dB0(Mh**2,Mh*Mh,Mh*Mh))/2. - (lfac*(pow((Lam3 + Lam4 - Lam5)*v1*(ZA22*ZA22)*ZH11 + 2*ZA21*(Lam1*v1*ZA21 + Lam5*v2*ZA22)*ZH11 + (Lam3 + Lam4 - Lam5)*v2*(ZA21*ZA21)*ZH12 + 2*ZA22*(Lam5*v1*ZA21 + Lam2*v2*ZA22)*ZH12,2)*loopfunctions.dB0(Mh**2,MA*MA,MA*MA) + pow((Lam3 + Lam4 - Lam5)*v1*(ZA12*ZA12)*ZH11 + 2*ZA11*(Lam1*v1*ZA11 + Lam5*v2*ZA12)*ZH11 + (Lam3 + Lam4 - Lam5)*v2*(ZA11*ZA11)*ZH12 + 2*ZA12*(Lam5*v1*ZA11 + Lam2*v2*ZA12)*ZH12,2)*loopfunctions.dB0(Mh**2,MG0*MG0,MG0*MG0) + pow(2*(2*Lam1*v1*ZH11 + Lam3*v2*ZH12)*ZP11*ZP21 + (Lam4 + Lam5)*(v2*ZH11 + v1*ZH12)*ZP12*ZP21 + v1*((Lam4 + Lam5)*ZH12*ZP11 + 2*Lam3*ZH11*ZP12)*ZP22 + v2*((Lam4 + Lam5)*ZH11*ZP11 + 4*Lam2*ZH12*ZP12)*ZP22,2)*loopfunctions.dB0(Mh**2,MGp*MGp,MHp**2) + pow((6*Lam1*v1*ZH11 + (Lam3 + Lam4 + Lam5)*v2*ZH12)*(ZH21*ZH21) + 2*(Lam3 + Lam4 + Lam5)*(v2*ZH11 + v1*ZH12)*ZH21*ZH22 + ((Lam3 + Lam4 + Lam5)*v1*ZH11 + 6*Lam2*v2*ZH12)*(ZH22*ZH22),2)*loopfunctions.dB0(Mh**2,MH**2,MH**2)))/2. - lfac*(pow((2*Lam1*v1*ZA11 + Lam5*v2*ZA12)*ZA21*ZH11 + (Lam5*v2*ZA11 + (Lam3 + Lam4 - Lam5)*v1*ZA12)*ZA22*ZH11 + Lam5*v1*(ZA12*ZA21 + ZA11*ZA22)*ZH12 + v2*((Lam3 + Lam4 - Lam5)*ZA11*ZA21 + 2*Lam2*ZA12*ZA22)*ZH12,2)*loopfunctions.dB0(Mh**2,MA*MA,MG0*MG0) + pow((2*Lam1*v1*ZH11 + Lam3*v2*ZH12)*(ZP11*ZP11) + (Lam4 + Lam5)*v2*ZH11*ZP11*ZP12 + (Lam4 + Lam5)*v1*ZH12*ZP11*ZP12 + Lam3*v1*ZH11*(ZP12*ZP12) + 2*Lam2*v2*ZH12*(ZP12*ZP12),2)*loopfunctions.dB0(Mh**2,MGp*MGp,MGp*MGp) + pow(6*Lam1*v1*(ZH11*ZH11)*ZH21 + (Lam3 + Lam4 + Lam5)*ZH12*(2*v2*ZH11 + v1*ZH12)*ZH21 + 6*Lam2*v2*(ZH12*ZH12)*ZH22 + (Lam3 + Lam4 + Lam5)*ZH11*(v2*ZH11 + 2*v1*ZH12)*ZH22,2)*loopfunctions.dB0(Mh**2,Mh*Mh,MH**2) + pow((2*Lam1*v1*ZH11 + Lam3*v2*ZH12)*(ZP21*ZP21) + (Lam4 + Lam5)*v2*ZH11*ZP21*ZP22 + (Lam4 + Lam5)*v1*ZH12*ZP21*ZP22 + Lam3*v1*ZH11*(ZP22*ZP22) + 2*Lam2*v2*ZH12*(ZP22*ZP22),2)*loopfunctions.dB0(Mh**2,MHp**2,MHp**2))
        hhselfBvFCderiv = (-2*alpha**2*lfac*(pi*pi)*pow(v1*ZH11 + v2*ZH12,2)*(8*(CW2*CW2)*loopfunctions.dB0(Mh**2,MWp*MWp,MWp*MWp) + pow(1 + CW2*CW2 + 2*CW2*SW2 + SW2*SW2,2)*loopfunctions.dB0(Mh**2,MZ**2,MZ**2)))/(CW2*CW2*(SW2*SW2))
        hhselfBsvFCderiv = (2*alpha*lfac*pi*(3*pow(ZH11*ZP11 + ZH12*ZP12,2)*loopfunctions.B0(Mh**2,MGp*MGp,MWp*MWp) + 3*pow(ZH11*ZP21 + ZH12*ZP22,2)*loopfunctions.B0(Mh**2,MHp**2,MWp*MWp) + 2*(pow(ZH11*ZP11 + ZH12*ZP12,2)*loopfunctions.B1(Mh**2,MWp*MWp,MGp*MGp) + pow(ZH11*ZP21 + ZH12*ZP22,2)*loopfunctions.B1(Mh**2,MWp*MWp,MHp**2)) + (MGp*MGp + 3*Mh**2)*pow(ZH11*ZP11 + ZH12*ZP12,2)*loopfunctions.dB0(Mh**2,MGp*MGp,MWp*MWp) + (MHp**2 + 3*Mh**2)*pow(ZH11*ZP21 + ZH12*ZP22,2)*loopfunctions.dB0(Mh**2,MHp**2,MWp*MWp) + 4*SW2*(pow(ZA21*ZH11 + ZA22*ZH12,2)*loopfunctions.B1(Mh**2,MZ**2,MA*MA) + pow(ZA11*ZH11 + ZA12*ZH12,2)*loopfunctions.B1(Mh**2,MZ**2,MG0*MG0))*pow(1/np.sin(2*np.arccos(sqrt(CW2))),2) + 2*SW2*(3*pow(ZA21*ZH11 + ZA22*ZH12,2)*loopfunctions.B0(Mh**2,MA*MA,MZ**2) + 3*pow(ZA11*ZH11 + ZA12*ZH12,2)*loopfunctions.B0(Mh**2,MG0*MG0,MZ**2) + (MA*MA + 3*Mh**2)*pow(ZA21*ZH11 + ZA22*ZH12,2)*loopfunctions.dB0(Mh**2,MA*MA,MZ**2) + (MG0*MG0 + 3*Mh**2)*pow(ZA11*ZH11 + ZA12*ZH12,2)*loopfunctions.dB0(Mh**2,MG0*MG0,MZ**2))*pow(1/np.sin(2*np.arccos(sqrt(CW2))),2) + 2*Mh**2*(pow(ZH11*ZP11 + ZH12*ZP12,2)*loopfunctions.dB1(Mh**2,MWp*MWp,MGp*MGp) + pow(ZH11*ZP21 + ZH12*ZP22,2)*loopfunctions.dB1(Mh**2,MWp*MWp,MHp**2) + 2*SW2*(pow(ZA21*ZH11 + ZA22*ZH12,2)*loopfunctions.dB1(Mh**2,MZ**2,MA*MA) + pow(ZA11*ZH11 + ZA12*ZH12,2)*loopfunctions.dB1(Mh**2,MZ**2,MG0*MG0))*pow(1/np.sin(2*np.arccos(sqrt(CW2))),2))))/SW2
        hHselfBtopFC = (-2*lfac*(Mt*Mt)*ZH12*ZH22*(-6*loopfunctions.A0(Mt*Mt) - 6*(2*(Mt*Mt)*loopfunctions.B0(pval*pval,Mt*Mt,Mt*Mt) + pval*pval*loopfunctions.B1(pval*pval,Mt*Mt,Mt*Mt))))/(v2*v2)
        hHselfBuFC = (alpha**2*lfac*(pi*pi)*(v1*ZH11 + v2*ZH12)*(v1*ZH21 + v2*ZH22)*(8*(CW2*CW2)*loopfunctions.B0(pval*pval,MWp*MWp,MWp*MWp) + pow(1 + CW2*CW2 + 2*CW2*SW2 + SW2*SW2,2)*loopfunctions.B0(pval*pval,MZ**2,MZ**2)))/(4.*(CW2*CW2)*(SW2*SW2))
        hHselfBsFC = - (lfac*(((Lam3 + Lam4 - Lam5)*v1*(ZA22*ZA22)*ZH11 + 2*ZA21*(Lam1*v1*ZA21 + Lam5*v2*ZA22)*ZH11 + (Lam3 + Lam4 - Lam5)*v2*(ZA21*ZA21)*ZH12 + 2*ZA22*(Lam5*v1*ZA21 + Lam2*v2*ZA22)*ZH12)*((Lam3 + Lam4 - Lam5)*v1*(ZA22*ZA22)*ZH21 + 2*ZA21*(Lam1*v1*ZA21 + Lam5*v2*ZA22)*ZH21 + (Lam3 + Lam4 - Lam5)*v2*(ZA21*ZA21)*ZH22 + 2*ZA22*(Lam5*v1*ZA21 + Lam2*v2*ZA22)*ZH22)*loopfunctions.B0(pval*pval,MA*MA,MA*MA) + ((Lam3 + Lam4 - Lam5)*v1*(ZA12*ZA12)*ZH11 + 2*ZA11*(Lam1*v1*ZA11 + Lam5*v2*ZA12)*ZH11 + (Lam3 + Lam4 - Lam5)*v2*(ZA11*ZA11)*ZH12 + 2*ZA12*(Lam5*v1*ZA11 + Lam2*v2*ZA12)*ZH12)*((Lam3 + Lam4 - Lam5)*v1*(ZA12*ZA12)*ZH21 + 2*ZA11*(Lam1*v1*ZA11 + Lam5*v2*ZA12)*ZH21 + (Lam3 + Lam4 - Lam5)*v2*(ZA11*ZA11)*ZH22 + 2*ZA12*(Lam5*v1*ZA11 + Lam2*v2*ZA12)*ZH22)*loopfunctions.B0(pval*pval,MG0*MG0,MG0*MG0) + (2*(2*Lam1*v1*ZH11 + Lam3*v2*ZH12)*ZP11*ZP21 + (Lam4 + Lam5)*(v2*ZH11 + v1*ZH12)*ZP12*ZP21 + v1*((Lam4 + Lam5)*ZH12*ZP11 + 2*Lam3*ZH11*ZP12)*ZP22 + v2*((Lam4 + Lam5)*ZH11*ZP11 + 4*Lam2*ZH12*ZP12)*ZP22)*(2*(2*Lam1*v1*ZH21 + Lam3*v2*ZH22)*ZP11*ZP21 + (Lam4 + Lam5)*(v2*ZH21 + v1*ZH22)*ZP12*ZP21 + v1*((Lam4 + Lam5)*ZH22*ZP11 + 2*Lam3*ZH21*ZP12)*ZP22 + v2*((Lam4 + Lam5)*ZH21*ZP11 + 4*Lam2*ZH22*ZP12)*ZP22)*loopfunctions.B0(pval*pval,MGp*MGp,MHp**2) + (6*Lam1*v1*(ZH11*ZH11)*ZH21 + (Lam3 + Lam4 + Lam5)*ZH12*(2*v2*ZH11 + v1*ZH12)*ZH21 + 6*Lam2*v2*(ZH12*ZH12)*ZH22 + (Lam3 + Lam4 + Lam5)*ZH11*(v2*ZH11 + 2*v1*ZH12)*ZH22)*(3*((Lam3 + Lam4 + Lam5)*ZH11*ZH12*(v2*ZH11 + v1*ZH12) + 2*(Lam1*v1*pow(ZH11,3) + Lam2*v2*pow(ZH12,3)))*loopfunctions.B0(pval*pval,Mh*Mh,Mh*Mh) + 2*((6*Lam1*v1*ZH11 + (Lam3 + Lam4 + Lam5)*v2*ZH12)*(ZH21*ZH21) + 2*(Lam3 + Lam4 + Lam5)*(v2*ZH11 + v1*ZH12)*ZH21*ZH22 + ((Lam3 + Lam4 + Lam5)*v1*ZH11 + 6*Lam2*v2*ZH12)*(ZH22*ZH22))*loopfunctions.B0(pval*pval,Mh*Mh,MH**2)) + 3*((6*Lam1*v1*ZH11 + (Lam3 + Lam4 + Lam5)*v2*ZH12)*(ZH21*ZH21) + 2*(Lam3 + Lam4 + Lam5)*(v2*ZH11 + v1*ZH12)*ZH21*ZH22 + ((Lam3 + Lam4 + Lam5)*v1*ZH11 + 6*Lam2*v2*ZH12)*(ZH22*ZH22))*((Lam3 + Lam4 + Lam5)*ZH21*ZH22*(v2*ZH21 + v1*ZH22) + 2*(Lam1*v1*pow(ZH21,3) + Lam2*v2*pow(ZH22,3)))*loopfunctions.B0(pval*pval,MH**2,MH**2) + 2*(((2*Lam1*v1*ZA11 + Lam5*v2*ZA12)*ZA21*ZH11 + (Lam5*v2*ZA11 + (Lam3 + Lam4 - Lam5)*v1*ZA12)*ZA22*ZH11 + Lam5*v1*(ZA12*ZA21 + ZA11*ZA22)*ZH12 + v2*((Lam3 + Lam4 - Lam5)*ZA11*ZA21 + 2*Lam2*ZA12*ZA22)*ZH12)*((2*Lam1*v1*ZA11 + Lam5*v2*ZA12)*ZA21*ZH21 + (Lam5*v2*ZA11 + (Lam3 + Lam4 - Lam5)*v1*ZA12)*ZA22*ZH21 + Lam5*v1*(ZA12*ZA21 + ZA11*ZA22)*ZH22 + v2*((Lam3 + Lam4 - Lam5)*ZA11*ZA21 + 2*Lam2*ZA12*ZA22)*ZH22)*loopfunctions.B0(pval*pval,MA*MA,MG0*MG0) + ((2*Lam1*v1*ZH11 + Lam3*v2*ZH12)*(ZP11*ZP11) + (Lam4 + Lam5)*v2*ZH11*ZP11*ZP12 + (Lam4 + Lam5)*v1*ZH12*ZP11*ZP12 + Lam3*v1*ZH11*(ZP12*ZP12) + 2*Lam2*v2*ZH12*(ZP12*ZP12))*((2*Lam1*v1*ZH21 + Lam3*v2*ZH22)*(ZP11*ZP11) + (Lam4 + Lam5)*v2*ZH21*ZP11*ZP12 + (Lam4 + Lam5)*v1*ZH22*ZP11*ZP12 + Lam3*v1*ZH21*(ZP12*ZP12) + 2*Lam2*v2*ZH22*(ZP12*ZP12))*loopfunctions.B0(pval*pval,MGp*MGp,MGp*MGp) + ((2*Lam1*v1*ZH11 + Lam3*v2*ZH12)*(ZP21*ZP21) + (Lam4 + Lam5)*v2*ZH11*ZP21*ZP22 + (Lam4 + Lam5)*v1*ZH12*ZP21*ZP22 + Lam3*v1*ZH11*(ZP22*ZP22) + 2*Lam2*v2*ZH12*(ZP22*ZP22))*((2*Lam1*v1*ZH21 + Lam3*v2*ZH22)*(ZP21*ZP21) + (Lam4 + Lam5)*v2*ZH21*ZP21*ZP22 + (Lam4 + Lam5)*v1*ZH22*ZP21*ZP22 + Lam3*v1*ZH21*(ZP22*ZP22) + 2*Lam2*v2*ZH22*(ZP22*ZP22))*loopfunctions.B0(pval*pval,MHp**2,MHp**2))))/2.
        hHselfBvFC = (alpha**2*lfac*(pi*pi)*(v1*ZH11 + v2*ZH12)*(v1*ZH21 + v2*ZH22)*(loopfunctions.MSDR*(pow(CW2,4) + SW2*SW2*pow(1 + SW2,2) + pow(CW2,3)*(2 + 4*SW2) + 2*CW2*SW2*(1 + 3*SW2 + 2*(SW2*SW2)) + CW2*CW2*(9 + 6*SW2 + 6*(SW2*SW2))) - 16*(CW2*CW2)*loopfunctions.B0(pval*pval,MWp*MWp,MWp*MWp) - 2*pow(1 + CW2*CW2 + 2*CW2*SW2 + SW2*SW2,2)*loopfunctions.B0(pval*pval,MZ**2,MZ**2)))/(CW2*CW2*(SW2*SW2))
        hHselfBsvFC = (2*alpha*lfac*pi*(((ZH11*ZP11 + ZH12*ZP12)*(ZH21*ZP11 + ZH22*ZP12) + (ZH11*ZP21 + ZH12*ZP22)*(ZH21*ZP21 + ZH22*ZP22))*loopfunctions.A0(MWp*MWp) + (MGp*MGp + 3*(pval*pval))*(ZH11*ZP11 + ZH12*ZP12)*(ZH21*ZP11 + ZH22*ZP12)*loopfunctions.B0(pval*pval,MGp*MGp,MWp*MWp) + (MHp**2 + 3*(pval*pval))*(ZH11*ZP21 + ZH12*ZP22)*(ZH21*ZP21 + ZH22*ZP22)*loopfunctions.B0(pval*pval,MHp**2,MWp*MWp) + 2*SW2*(((ZA11*ZH11 + ZA12*ZH12)*(ZA11*ZH21 + ZA12*ZH22) + (ZA21*ZH11 + ZA22*ZH12)*(ZA21*ZH21 + ZA22*ZH22))*loopfunctions.A0(MZ**2) + (MA*MA + 3*(pval*pval))*(ZA21*ZH11 + ZA22*ZH12)*(ZA21*ZH21 + ZA22*ZH22)*loopfunctions.B0(pval*pval,MA*MA,MZ**2) + (MG0*MG0 + 3*(pval*pval))*(ZA11*ZH11 + ZA12*ZH12)*(ZA11*ZH21 + ZA12*ZH22)*loopfunctions.B0(pval*pval,MG0*MG0,MZ**2))*pow(1/np.sin(2*np.arccos(sqrt(CW2))),2) + 2*(pval*pval)*((ZH11*ZP11 + ZH12*ZP12)*(ZH21*ZP11 + ZH22*ZP12)*loopfunctions.B1(pval*pval,MWp*MWp,MGp*MGp) + (ZH11*ZP21 + ZH12*ZP22)*(ZH21*ZP21 + ZH22*ZP22)*loopfunctions.B1(pval*pval,MWp*MWp,MHp**2) + 2*SW2*((ZA21*ZH11 + ZA22*ZH12)*(ZA21*ZH21 + ZA22*ZH22)*loopfunctions.B1(pval*pval,MZ**2,MA*MA) + (ZA11*ZH11 + ZA12*ZH12)*(ZA11*ZH21 + ZA12*ZH22)*loopfunctions.B1(pval*pval,MZ**2,MG0*MG0))*pow(1/np.sin(2*np.arccos(sqrt(CW2))),2))))/SW2
        hhselfBtotFCderiv = hhselfBtopFCderiv+hhselfBsFCderiv+hhselfBsvFCderiv+hhselfBvFCderiv+hhselfBuFCderiv
        hHselfBtotFC = hHselfBsFC+hHselfBsvFC+hHselfBvFC+hHselfBuFC+hHselfBtopFC
        test.assert_allclose(wfrtB, 3/2*lamhhh*hhselfBtotFCderiv - 3*lamhhH*hHselfBtotFC/(Mh**2-MH**2))

    def test_WFR_tad(self):
        wfrtTA = THDM.process(higgs,higgs,higgs,momenta=[Mh**2,Mh**2,Mh**2],exclude_particles=['d1','d2','d3','u1','u2','e1','e2','e3'], only_topologies=['ThreePointWFRT'])['ThreePointWFRT']
        lamhhH = - (-((6*Lam1*v1*(ZH11*ZH11) + (Lam3 + Lam4 + Lam5)*ZH12*(2*v2*ZH11 + v1*ZH12))*ZH21) - (6*Lam2*v2*(ZH12*ZH12) + (Lam3 + Lam4 + Lam5)*ZH11*(v2*ZH11 + 2*v1*ZH12))*ZH22)
        hHselfTAtopFC = - 12*lfac*Mt**2/v2*loopfunctions.A0(Mt*Mt)*(ZH12*(6*Lam1*v1*(ZH11*ZH11)*ZH21 + (Lam3 + Lam4 + Lam5)*ZH12*(2*v2*ZH11 + v1*ZH12)*ZH21 + 6*Lam2*v2*(ZH12*ZH12)*ZH22 + (Lam3 + Lam4 + Lam5)*ZH11*(v2*ZH11 + 2*v1*ZH12)*ZH22)/(Mh*Mh) + ZH22*((6*Lam1*v1*ZH11 + (Lam3 + Lam4 + Lam5)*v2*ZH12)*(ZH21*ZH21) + 2*(Lam3 + Lam4 + Lam5)*(v2*ZH11 + v1*ZH12)*ZH21*ZH22 + ((Lam3 + Lam4 + Lam5)*v1*ZH11 + 6*Lam2*v2*ZH12)*(ZH22*ZH22))/(MH**2))
        hHselfTAsvuFC = (lfac*(MH**2*(6*Lam1*v1*(ZH11*ZH11)*ZH21 + (Lam3 + Lam4 + Lam5)*ZH12*(2*v2*ZH11 + v1*ZH12)*ZH21 + 6*Lam2*v2*(ZH12*ZH12)*ZH22 + (Lam3 + Lam4 + Lam5)*ZH11*(v2*ZH11 + 2*v1*ZH12)*ZH22)*(CW2*SW2*(((Lam3 + Lam4 - Lam5)*v1*(ZA22*ZA22)*ZH11 + 2*ZA21*(Lam1*v1*ZA21 + Lam5*v2*ZA22)*ZH11 + (Lam3 + Lam4 - Lam5)*v2*(ZA21*ZA21)*ZH12 + 2*ZA22*(Lam5*v1*ZA21 + Lam2*v2*ZA22)*ZH12)*loopfunctions.A0(MA*MA) + ((Lam3 + Lam4 - Lam5)*v1*(ZA12*ZA12)*ZH11 + 2*ZA11*(Lam1*v1*ZA11 + Lam5*v2*ZA12)*ZH11 + (Lam3 + Lam4 - Lam5)*v2*(ZA11*ZA11)*ZH12 + 2*ZA12*(Lam5*v1*ZA11 + Lam2*v2*ZA12)*ZH12)*loopfunctions.A0(MG0*MG0)) + 3*CW2*SW2*((Lam3 + Lam4 + Lam5)*ZH11*ZH12*(v2*ZH11 + v1*ZH12) + 2*(Lam1*v1*pow(ZH11,3) + Lam2*v2*pow(ZH12,3)))*loopfunctions.A0(Mh*Mh) + CW2*SW2*((6*Lam1*v1*ZH11 + (Lam3 + Lam4 + Lam5)*v2*ZH12)*(ZH21*ZH21) + 2*(Lam3 + Lam4 + Lam5)*(v2*ZH11 + v1*ZH12)*ZH21*ZH22 + ((Lam3 + Lam4 + Lam5)*v1*ZH11 + 6*Lam2*v2*ZH12)*(ZH22*ZH22))*loopfunctions.A0(MH**2) + 2*CW2*SW2*(((2*Lam1*v1*ZH11 + Lam3*v2*ZH12)*(ZP11*ZP11) + (Lam4 + Lam5)*v2*ZH11*ZP11*ZP12 + (Lam4 + Lam5)*v1*ZH12*ZP11*ZP12 + Lam3*v1*ZH11*(ZP12*ZP12) + 2*Lam2*v2*ZH12*(ZP12*ZP12))*loopfunctions.A0(MGp*MGp) + ((2*Lam1*v1*ZH11 + Lam3*v2*ZH12)*(ZP21*ZP21) + (Lam4 + Lam5)*v2*ZH11*ZP21*ZP22 + (Lam4 + Lam5)*v1*ZH12*ZP21*ZP22 + Lam3*v1*ZH11*(ZP22*ZP22) + 2*Lam2*v2*ZH12*(ZP22*ZP22))*loopfunctions.A0(MHp**2)) - alpha*pi*(v1*ZH11 + v2*ZH12)*(2*loopfunctions.MSDR*(CW2*CW2*MZ**2 + MZ**2*SW2*(1 + SW2) + CW2*(4*(MWp*MWp) + MZ**2 + 2*MZ**2*SW2)) - 12*CW2*loopfunctions.A0(MWp*MWp) - 3*(1 + CW2*CW2 + 2*CW2*SW2 + SW2*SW2)*loopfunctions.A0(MZ**2))) + Mh*Mh*((6*Lam1*v1*ZH11 + (Lam3 + Lam4 + Lam5)*v2*ZH12)*(ZH21*ZH21) + 2*(Lam3 + Lam4 + Lam5)*(v2*ZH11 + v1*ZH12)*ZH21*ZH22 + ((Lam3 + Lam4 + Lam5)*v1*ZH11 + 6*Lam2*v2*ZH12)*(ZH22*ZH22))*(CW2*SW2*(((Lam3 + Lam4 - Lam5)*v1*(ZA22*ZA22)*ZH21 + 2*ZA21*(Lam1*v1*ZA21 + Lam5*v2*ZA22)*ZH21 + (Lam3 + Lam4 - Lam5)*v2*(ZA21*ZA21)*ZH22 + 2*ZA22*(Lam5*v1*ZA21 + Lam2*v2*ZA22)*ZH22)*loopfunctions.A0(MA*MA) + ((Lam3 + Lam4 - Lam5)*v1*(ZA12*ZA12)*ZH21 + 2*ZA11*(Lam1*v1*ZA11 + Lam5*v2*ZA12)*ZH21 + (Lam3 + Lam4 - Lam5)*v2*(ZA11*ZA11)*ZH22 + 2*ZA12*(Lam5*v1*ZA11 + Lam2*v2*ZA12)*ZH22)*loopfunctions.A0(MG0*MG0) + (6*Lam1*v1*(ZH11*ZH11)*ZH21 + (Lam3 + Lam4 + Lam5)*ZH12*(2*v2*ZH11 + v1*ZH12)*ZH21 + 6*Lam2*v2*(ZH12*ZH12)*ZH22 + (Lam3 + Lam4 + Lam5)*ZH11*(v2*ZH11 + 2*v1*ZH12)*ZH22)*loopfunctions.A0(Mh*Mh)) + 3*CW2*SW2*((Lam3 + Lam4 + Lam5)*ZH21*ZH22*(v2*ZH21 + v1*ZH22) + 2*(Lam1*v1*pow(ZH21,3) + Lam2*v2*pow(ZH22,3)))*loopfunctions.A0(MH**2) + 2*CW2*SW2*(((2*Lam1*v1*ZH21 + Lam3*v2*ZH22)*(ZP11*ZP11) + (Lam4 + Lam5)*v2*ZH21*ZP11*ZP12 + (Lam4 + Lam5)*v1*ZH22*ZP11*ZP12 + Lam3*v1*ZH21*(ZP12*ZP12) + 2*Lam2*v2*ZH22*(ZP12*ZP12))*loopfunctions.A0(MGp*MGp) + ((2*Lam1*v1*ZH21 + Lam3*v2*ZH22)*(ZP21*ZP21) + (Lam4 + Lam5)*v2*ZH21*ZP21*ZP22 + (Lam4 + Lam5)*v1*ZH22*ZP21*ZP22 + Lam3*v1*ZH21*(ZP22*ZP22) + 2*Lam2*v2*ZH22*(ZP22*ZP22))*loopfunctions.A0(MHp**2)) - alpha*pi*(v1*ZH21 + v2*ZH22)*(2*loopfunctions.MSDR*(CW2*CW2*MZ**2 + MZ**2*SW2*(1 + SW2) + CW2*(4*(MWp*MWp) + MZ**2 + 2*MZ**2*SW2)) - 12*CW2*loopfunctions.A0(MWp*MWp) - 3*(1 + CW2*CW2 + 2*CW2*SW2 + SW2*SW2)*loopfunctions.A0(MZ**2)))))/(2.*CW2*(Mh*Mh)*MH**2*SW2)
        test.assert_allclose(wfrtTA, - 3*lamhhH*(hHselfTAtopFC+hHselfTAsvuFC)/(Mh**2-MH**2))

    """ test charged Higgs self-energy """
    def test_HpHm_A(self):
        HpHmselfsanyH3A = THDM.process(p['Hm2'],p['Hm2c'],momenta=[MHp**2],exclude_particles=['u1','d1','u2','d2','e3','nu1','nu2','nu3'], only_topologies=['TwoPointA'])['TwoPointA']
        HpHmselfAFC = (lfac*(-(((2*Lam1*(ZA21*ZA21) + Lam3*(ZA22*ZA22))*(ZP21*ZP21) + 2*(Lam4 + Lam5)*ZA21*ZA22*ZP21*ZP22 + (Lam3*(ZA21*ZA21) + 2*Lam2*(ZA22*ZA22))*(ZP22*ZP22))*loopfunctions.A0(MA*MA)) - ((2*Lam1*(ZA11*ZA11) + Lam3*(ZA12*ZA12))*(ZP21*ZP21) + 2*(Lam4 + Lam5)*ZA11*ZA12*ZP21*ZP22 + (Lam3*(ZA11*ZA11) + 2*Lam2*(ZA12*ZA12))*(ZP22*ZP22))*loopfunctions.A0(MG0*MG0) - 2*((4*Lam1*(ZP11*ZP11) + (Lam3 + Lam4)*(ZP12*ZP12))*(ZP21*ZP21) + 2*(Lam3 + Lam4 + 2*Lam5)*ZP11*ZP12*ZP21*ZP22 + ((Lam3 + Lam4)*(ZP11*ZP11) + 4*Lam2*(ZP12*ZP12))*(ZP22*ZP22))*loopfunctions.A0(MGp*MGp) - ((2*Lam1*(ZH11*ZH11) + Lam3*(ZH12*ZH12))*(ZP21*ZP21) + 2*(Lam4 + Lam5)*ZH11*ZH12*ZP21*ZP22 + (Lam3*(ZH11*ZH11) + 2*Lam2*(ZH12*ZH12))*(ZP22*ZP22))*loopfunctions.A0(Mh*Mh) - ((2*Lam1*(ZH21*ZH21) + Lam3*(ZH22*ZH22))*(ZP21*ZP21) + 2*(Lam4 + Lam5)*ZH21*ZH22*ZP21*ZP22 + (Lam3*(ZH21*ZH21) + 2*Lam2*(ZH22*ZH22))*(ZP22*ZP22))*loopfunctions.A0(MH**2) - 8*(Lam1*pow(ZP21,4) + (Lam3 + Lam4 + Lam5)*(ZP21*ZP21)*(ZP22*ZP22) + Lam2*pow(ZP22,4))*loopfunctions.A0(MHp**2) + (2*alpha*pi*(ZP21*ZP21 + ZP22*ZP22)*(loopfunctions.MSDR*(CW2*CW2*MZ**2 + MZ**2*SW2*(1 + SW2) + CW2*(4*(MWp*MWp) + MZ**2 - 6*MZ**2*SW2)) - 8*CW2*loopfunctions.A0(MWp*MWp) - 2*(1 + CW2*CW2 - 6*CW2*SW2 + SW2*SW2)*loopfunctions.A0(MZ**2)))/(CW2*SW2)))/2.
        test.assert_allclose(HpHmselfsanyH3A, HpHmselfAFC)

    def test_HpHm_B(self):
        HpHmselfsanyH3B = THDM.process(p['Hm2'],p['Hm2c'],momenta=[MHp**2],exclude_particles=['u1','d1','u2','d2','e3','nu1','nu2','nu3'], only_topologies=['TwoPointB'])['TwoPointB']
        HpHmselfBtbFC = 12*lfac*pow(v1,-2)*pow(v2,-2)*(loopfunctions.A0(pow(Mb,2))*(pow(Mb,2)*pow(v2,2)*pow(ZP21,2) + pow(Mt,2)*pow(v1,2)*pow(ZP22,2)) - MHp**2*loopfunctions.B1(MHp**2,pow(Mb,2),pow(Mt,2))*(pow(Mb,2)*pow(v2,2)*pow(ZP21,2) + pow(Mt,2)*pow(v1,2)*pow(ZP22,2)) + loopfunctions.B0(MHp**2,pow(Mb,2),pow(Mt,2))*(v2*ZP21*pow(Mb,2)*(-(MHp**2*v2*ZP21) + (v2*ZP21 - 2*v1*ZP22)*pow(Mt,2)) + pow(Mt,2)*(-MHp**2 + pow(Mt,2))*pow(v1,2)*pow(ZP22,2)))
        HpHmselfBsFC = - (lfac*(pow(Lam4 - Lam5,2)*(loopfunctions.B0(MHp**2,pow(MG0,2),pow(MGp,2))*pow(v2*ZA11 - v1*ZA12,2) + loopfunctions.B0(MHp**2,pow(MA,2),pow(MGp,2))*pow(v2*ZA21 - v1*ZA22,2))*pow(ZP12*ZP21 - ZP11*ZP22,2) + loopfunctions.B0(MHp**2,pow(MGp,2),pow(Mh,2))*pow(2*(2*Lam1*v1*ZH11 + Lam3*v2*ZH12)*ZP11*ZP21 + (Lam4 + Lam5)*(v2*ZH11 + v1*ZH12)*ZP12*ZP21 + v1*((Lam4 + Lam5)*ZH12*ZP11 + 2*Lam3*ZH11*ZP12)*ZP22 + v2*((Lam4 + Lam5)*ZH11*ZP11 + 4*Lam2*ZH12*ZP12)*ZP22,2) + loopfunctions.B0(MHp**2,pow(MGp,2),MH**2)*pow(2*(2*Lam1*v1*ZH21 + Lam3*v2*ZH22)*ZP11*ZP21 + (Lam4 + Lam5)*(v2*ZH21 + v1*ZH22)*ZP12*ZP21 + v1*((Lam4 + Lam5)*ZH22*ZP11 + 2*Lam3*ZH21*ZP12)*ZP22 + v2*((Lam4 + Lam5)*ZH21*ZP11 + 4*Lam2*ZH22*ZP12)*ZP22,2)))/4. - lfac*(loopfunctions.B0(MHp**2,pow(Mh,2),MHp**2)*pow((Lam4 + Lam5)*v2*ZH11*ZP21*ZP22 + (Lam4 + Lam5)*v1*ZH12*ZP21*ZP22 + (2*Lam1*v1*ZH11 + Lam3*v2*ZH12)*pow(ZP21,2) + Lam3*v1*ZH11*pow(ZP22,2) + 2*Lam2*v2*ZH12*pow(ZP22,2),2) + loopfunctions.B0(MHp**2,MH**2,MHp**2)*pow((Lam4 + Lam5)*v2*ZH21*ZP21*ZP22 + (Lam4 + Lam5)*v1*ZH22*ZP21*ZP22 + (2*Lam1*v1*ZH21 + Lam3*v2*ZH22)*pow(ZP21,2) + Lam3*v1*ZH21*pow(ZP22,2) + 2*Lam2*v2*ZH22*pow(ZP22,2),2))
        HpHmselfBvFC = 8*alpha**2*lfac*(loopfunctions.MSDR - 2*CW2*loopfunctions.B0(MHp**2,0,pow(MWp,2)) - 2*SW2*loopfunctions.B0(MHp**2,pow(MWp,2),MZ**2))*pow(CW2,-1)*pow(pi,2)*pow(SW2,-1)*pow(v1*ZP21 + v2*ZP22,2)
        HpHmselfBsvFC = (lfac*(4*alpha*pi*pow(SW2,-1)*(loopfunctions.B0(MHp**2,pow(MG0,2),pow(MWp,2))*(3*MHp**2 + pow(MG0,2))*pow(ZA11*ZP21 + ZA12*ZP22,2) + loopfunctions.B0(MHp**2,pow(MA,2),pow(MWp,2))*(3*MHp**2 + pow(MA,2))*pow(ZA21*ZP21 + ZA22*ZP22,2) + loopfunctions.B0(MHp**2,pow(Mh,2),pow(MWp,2))*(3*MHp**2 + pow(Mh,2))*pow(ZH11*ZP21 + ZH12*ZP22,2) + (MH**2 + 3*MHp**2)*loopfunctions.B0(MHp**2,MH**2,pow(MWp,2))*pow(ZH21*ZP21 + ZH22*ZP22,2) + loopfunctions.A0(pow(MWp,2))*(pow(ZA11*ZP21 + ZA12*ZP22,2) + pow(ZA21*ZP21 + ZA22*ZP22,2) + pow(ZH11*ZP21 + ZH12*ZP22,2) + pow(ZH21*ZP21 + ZH22*ZP22,2))) + loopfunctions.A0(MZ**2)*(pow(ZP11*ZP21 + ZP12*ZP22,2) + pow(pow(ZP21,2) + pow(ZP22,2),2))*pow(EL/np.tan(np.arccos(sqrt(CW2))) - EL*np.tan(np.arccos(sqrt(CW2))),2) + 4*MHp**2*pow(pow(ZP21,2) + pow(ZP22,2),2)*(16*alpha*pi*loopfunctions.B0(MHp**2,0,MHp**2) + loopfunctions.B0(MHp**2,MHp**2,MZ**2)*pow(EL/np.tan(np.arccos(sqrt(CW2)))  - EL*np.tan(np.arccos(sqrt(CW2))),2)) + (3*MHp**2 + pow(MGp,2))*pow(ZP11*ZP21 + ZP12*ZP22,2)*(16*alpha*pi*loopfunctions.B0(MHp**2,0,pow(MGp,2)) + loopfunctions.B0(MHp**2,pow(MGp,2),MZ**2)*pow(EL/np.tan(np.arccos(sqrt(CW2)))  - EL*np.tan(np.arccos(sqrt(CW2))),2)) + 2*MHp**2*(4*alpha*pi*pow(SW2,-1)*(loopfunctions.B1(MHp**2,pow(MWp,2),pow(MG0,2))*pow(ZA11*ZP21 + ZA12*ZP22,2) + loopfunctions.B1(MHp**2,pow(MWp,2),pow(MA,2))*pow(ZA21*ZP21 + ZA22*ZP22,2) + loopfunctions.B1(MHp**2,pow(MWp,2),pow(Mh,2))*pow(ZH11*ZP21 + ZH12*ZP22,2) + loopfunctions.B1(MHp**2,pow(MWp,2),MH**2)*pow(ZH21*ZP21 + ZH22*ZP22,2)) + 16*alpha*pi*(loopfunctions.B1(MHp**2,0,pow(MGp,2))*pow(ZP11*ZP21 + ZP12*ZP22,2) + loopfunctions.B1(MHp**2,0,MHp**2)*pow(pow(ZP21,2) + pow(ZP22,2),2)) + (loopfunctions.B1(MHp**2,MZ**2,pow(MGp,2))*pow(ZP11*ZP21 + ZP12*ZP22,2) + loopfunctions.B1(MHp**2,MZ**2,MHp**2)*pow(pow(ZP21,2) + pow(ZP22,2),2))*pow(EL/np.tan(np.arccos(sqrt(CW2))) - EL*np.tan(np.arccos(sqrt(CW2))),2))))/4.
        HpHmselfBuFC = (-8*alpha**2*lfac*(pi*pi)*pow(v1*ZP21 + v2*ZP22,2)*loopfunctions.B0(MHp**2,MWp*MWp,MZ**2)*1/np.tan(2*np.arccos(sqrt(CW2)))*1/np.sin(2*np.arccos(sqrt(CW2))))/SW2
        test.assert_allclose(HpHmselfsanyH3B, HpHmselfBtbFC+HpHmselfBsFC+HpHmselfBsvFC+HpHmselfBvFC+HpHmselfBuFC)

    def test_HpHm_tad(self):
        HpHmselfsanyH3TA = THDM.process(p['Hm2'],p['Hm2c'],momenta=[MHp**2],exclude_particles=['u1','d1','u2','d2','u1bar','d1bar','u2bar','d2bar','e1','e2','e3','e1bar','e2bar','e3bar','nu1','nu2','nu3', 'nu1bar', 'nu2bar', 'nu3bar','d3','d3bar'], only_topologies=['TwoPointTA'])['TwoPointTA']
        HpHmselfTAtopFC = - 12*lfac*Mt**2/v2*((ZH12*((2*Lam1*v1*ZH11 + Lam3*v2*ZH12)*(ZP21*ZP21) + (Lam4 + Lam5)*v2*ZH11*ZP21*ZP22 + (Lam4 + Lam5)*v1*ZH12*ZP21*ZP22 + Lam3*v1*ZH11*(ZP22*ZP22) + 2*Lam2*v2*ZH12*(ZP22*ZP22)))/(Mh*Mh) + (ZH22*((2*Lam1*v1*ZH21 + Lam3*v2*ZH22)*(ZP21*ZP21) + (Lam4 + Lam5)*v2*ZH21*ZP21*ZP22 + (Lam4 + Lam5)*v1*ZH22*ZP21*ZP22 + Lam3*v1*ZH21*(ZP22*ZP22) + 2*Lam2*v2*ZH22*(ZP22*ZP22)))/MH**2)*loopfunctions.A0(Mt*Mt)
        HpHmselfTAsvuFC = (lfac*(MH**2*((2*Lam1*v1*ZH11 + Lam3*v2*ZH12)*(ZP21*ZP21) + (Lam4 + Lam5)*v2*ZH11*ZP21*ZP22 + (Lam4 + Lam5)*v1*ZH12*ZP21*ZP22 + Lam3*v1*ZH11*(ZP22*ZP22) + 2*Lam2*v2*ZH12*(ZP22*ZP22))*(CW2*SW2*(((Lam3 + Lam4 - Lam5)*v1*(ZA22*ZA22)*ZH11 + 2*ZA21*(Lam1*v1*ZA21 + Lam5*v2*ZA22)*ZH11 + (Lam3 + Lam4 - Lam5)*v2*(ZA21*ZA21)*ZH12 + 2*ZA22*(Lam5*v1*ZA21 + Lam2*v2*ZA22)*ZH12)*loopfunctions.A0(MA*MA) + ((Lam3 + Lam4 - Lam5)*v1*(ZA12*ZA12)*ZH11 + 2*ZA11*(Lam1*v1*ZA11 + Lam5*v2*ZA12)*ZH11 + (Lam3 + Lam4 - Lam5)*v2*(ZA11*ZA11)*ZH12 + 2*ZA12*(Lam5*v1*ZA11 + Lam2*v2*ZA12)*ZH12)*loopfunctions.A0(MG0*MG0)) + 2*CW2*SW2*((2*Lam1*v1*ZH11 + Lam3*v2*ZH12)*(ZP11*ZP11) + (Lam4 + Lam5)*v2*ZH11*ZP11*ZP12 + (Lam4 + Lam5)*v1*ZH12*ZP11*ZP12 + Lam3*v1*ZH11*(ZP12*ZP12) + 2*Lam2*v2*ZH12*(ZP12*ZP12))*loopfunctions.A0(MGp*MGp) + 3*CW2*SW2*((Lam3 + Lam4 + Lam5)*ZH11*ZH12*(v2*ZH11 + v1*ZH12) + 2*(Lam1*v1*pow(ZH11,3) + Lam2*v2*pow(ZH12,3)))*loopfunctions.A0(Mh*Mh) + CW2*SW2*((6*Lam1*v1*ZH11 + (Lam3 + Lam4 + Lam5)*v2*ZH12)*(ZH21*ZH21) + 2*(Lam3 + Lam4 + Lam5)*(v2*ZH11 + v1*ZH12)*ZH21*ZH22 + ((Lam3 + Lam4 + Lam5)*v1*ZH11 + 6*Lam2*v2*ZH12)*(ZH22*ZH22))*loopfunctions.A0(MH**2) + 2*CW2*SW2*((2*Lam1*v1*ZH11 + Lam3*v2*ZH12)*(ZP21*ZP21) + (Lam4 + Lam5)*v2*ZH11*ZP21*ZP22 + (Lam4 + Lam5)*v1*ZH12*ZP21*ZP22 + Lam3*v1*ZH11*(ZP22*ZP22) + 2*Lam2*v2*ZH12*(ZP22*ZP22))*loopfunctions.A0(MHp**2) - alpha*pi*(v1*ZH11 + v2*ZH12)*(2*loopfunctions.MSDR*(CW2*CW2*MZ**2 + MZ**2*SW2*(1 + SW2) + CW2*(4*(MWp*MWp) + MZ**2 + 2*MZ**2*SW2)) - 12*CW2*loopfunctions.A0(MWp*MWp) - 3*(1 + CW2*CW2 + 2*CW2*SW2 + SW2*SW2)*loopfunctions.A0(MZ**2))) + Mh*Mh*((2*Lam1*v1*ZH21 + Lam3*v2*ZH22)*(ZP21*ZP21) + (Lam4 + Lam5)*v2*ZH21*ZP21*ZP22 + (Lam4 + Lam5)*v1*ZH22*ZP21*ZP22 + Lam3*v1*ZH21*(ZP22*ZP22) + 2*Lam2*v2*ZH22*(ZP22*ZP22))*(2*CW2*SW2*((2*Lam1*v1*ZH21 + Lam3*v2*ZH22)*(ZP11*ZP11) + (Lam4 + Lam5)*v2*ZH21*ZP11*ZP12 + (Lam4 + Lam5)*v1*ZH22*ZP11*ZP12 + Lam3*v1*ZH21*(ZP12*ZP12) + 2*Lam2*v2*ZH22*(ZP12*ZP12))*loopfunctions.A0(MGp*MGp) + CW2*SW2*(((Lam3 + Lam4 - Lam5)*v1*(ZA22*ZA22)*ZH21 + 2*ZA21*(Lam1*v1*ZA21 + Lam5*v2*ZA22)*ZH21 + (Lam3 + Lam4 - Lam5)*v2*(ZA21*ZA21)*ZH22 + 2*ZA22*(Lam5*v1*ZA21 + Lam2*v2*ZA22)*ZH22)*loopfunctions.A0(MA*MA) + ((Lam3 + Lam4 - Lam5)*v1*(ZA12*ZA12)*ZH21 + 2*ZA11*(Lam1*v1*ZA11 + Lam5*v2*ZA12)*ZH21 + (Lam3 + Lam4 - Lam5)*v2*(ZA11*ZA11)*ZH22 + 2*ZA12*(Lam5*v1*ZA11 + Lam2*v2*ZA12)*ZH22)*loopfunctions.A0(MG0*MG0) + (6*Lam1*v1*(ZH11*ZH11)*ZH21 + (Lam3 + Lam4 + Lam5)*ZH12*(2*v2*ZH11 + v1*ZH12)*ZH21 + 6*Lam2*v2*(ZH12*ZH12)*ZH22 + (Lam3 + Lam4 + Lam5)*ZH11*(v2*ZH11 + 2*v1*ZH12)*ZH22)*loopfunctions.A0(Mh*Mh)) + 3*CW2*SW2*((Lam3 + Lam4 + Lam5)*ZH21*ZH22*(v2*ZH21 + v1*ZH22) + 2*(Lam1*v1*pow(ZH21,3) + Lam2*v2*pow(ZH22,3)))*loopfunctions.A0(MH**2) + 2*CW2*SW2*((2*Lam1*v1*ZH21 + Lam3*v2*ZH22)*(ZP21*ZP21) + (Lam4 + Lam5)*v2*ZH21*ZP21*ZP22 + (Lam4 + Lam5)*v1*ZH22*ZP21*ZP22 + Lam3*v1*ZH21*(ZP22*ZP22) + 2*Lam2*v2*ZH22*(ZP22*ZP22))*loopfunctions.A0(MHp**2) - alpha*pi*(v1*ZH21 + v2*ZH22)*(2*loopfunctions.MSDR*(CW2*CW2*MZ**2 + MZ**2*SW2*(1 + SW2) + CW2*(4*(MWp*MWp) + MZ**2 + 2*MZ**2*SW2)) - 12*CW2*loopfunctions.A0(MWp*MWp) - 3*(1 + CW2*CW2 + 2*CW2*SW2 + SW2*SW2)*loopfunctions.A0(MZ**2)))))/(2.*CW2*(Mh*Mh)*MH**2*SW2)
        test.assert_allclose(HpHmselfsanyH3TA, HpHmselfTAtopFC+HpHmselfTAsvuFC)
